############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5
from baseclasses import widget
from gui import message_window_ui


class MessageWindow(widget.MwWidget):
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        super(MessageWindow, self).__init__()
        self.app = app
        self.showStatus = False
        self.ui = message_window_ui.Ui_MessageDialog()
        self.ui.setupUi(self)
        self.initUI()
        # allow sizing of the window
        self.setFixedSize(PyQt5.QtCore.QSize(16777215, 16777215))
        # self.setSizePolicy(PyQt5.QtWidgets.QSizePolicy.Pref, PyQt5.QtWidgets.QSizePolicy.Ignored)
        self.setMinimumSize(790, 200)
        self.setMaximumSize(790, 16777215)
        self.initConfig()

    def resizeEvent(self, QResizeEvent):
        # allow message window to be resized in height
        self.ui.messages.setGeometry(10, 10, 771, self.height() - 20)

    def initConfig(self):
        try:
            if 'MessageWindowPositionX' in self.app.config:
                x = self.app.config['MessageWindowPositionX']
                y = self.app.config['MessageWindowPositionY']
                if x > self.screenSizeX:
                    x = 0
                if y > self.screenSizeY:
                    y = 0
                self.move(x, y)
            if 'MessageWindowShowStatus' in self.app.config:
                self.showStatus = self.app.config['MessageWindowShowStatus']
            if 'MessageWindowHeight' in self.app.config:
                self.resize(791, self.app.config['MessageWindowHeight'])
        except Exception as e:
            self.logger.error('Item in config.cfg not be initialized for messages window, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['MessageWindowPositionX'] = self.pos().x()
        self.app.config['MessageWindowPositionY'] = self.pos().y()
        self.app.config['MessageWindowShowStatus'] = self.showStatus
        self.app.config['MessageWindowHeight'] = self.height()

    def toggleWindow(self):
        self.showStatus = not self.showStatus
        if self.showStatus:
            self.showWindow()
        else:
            self.close()

    def showWindow(self):
        self.showStatus = True
        self.setVisible(True)
        self.show()
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openMessageWindow, 'running', 'true')

    def closeEvent(self, closeEvent):
        super().closeEvent(closeEvent)
        self.app.signalChangeStylesheet.emit(self.app.ui.btn_openMessageWindow, 'running', 'false')



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import json
import logging
import platform
import time
import PyQt5
import requests
from baseclasses import checkIP


class SGPro:
    logger = logging.getLogger(__name__)

    # base definitions of class
    host = '127.0.0.1'
    port = 59590
    ipSGProBase = 'http://' + host + ':' + str(port)
    ipSGPro = 'http://' + host + ':' + str(port) + '/json/reply/'
    captureImagePath = 'SgCaptureImage'
    getCameraPropsPath = 'SgGetCameraProps'
    getDeviceStatusPath = 'SgGetDeviceStatus'
    getImagePath = 'SgGetImagePath'

    CAMERA_STATUS = {'ERROR': 'ERROR', 'DISCONNECTED': 'DISCONNECTED', 'BUSY': 'DOWNLOADING', 'READY': 'DOWNLOADING', 'IDLE': 'IDLE', 'INTEGRATING': 'INTEGRATING'}

    def __init__(self, main, app, data):
        # make main sources available
        self.main = main
        self.app = app
        self.data = data
        self.cancel = False
        self.checkIP = checkIP.CheckIP()
        self.mutexCancel = PyQt5.QtCore.QMutex()

        self.application = dict()
        self.application['Available'] = False
        self.application['Name'] = ''
        self.application['InstallPath'] = ''
        self.application['Status'] = ''
        self.application['Runtime'] = 'Sequence Generator.exe'

        if platform.system() == 'Windows':
            # sgpro only supported on local machine
            self.application['Available'], self.application['Name'], self.application['InstallPath'] = self.app.checkRegistrationKeys('Sequence Generator')
            if self.application['Available']:
                self.app.messageQueue.put('Found Imaging: {0}\n'.format(self.application['Name']))
                self.logger.info('Name: {0}, Path: {1}'.format(self.application['Name'], self.application['InstallPath']))
            else:
                self.logger.info('Application SGPro not found on computer')

    def start(self):
        pass

    def stop(self):
        pass

    def getStatus(self):
        if self.checkIP.checkIPAvailable(self.host, self.port):
            suc, state, message = self.SgGetDeviceStatus('Camera')
            if suc:
                self.application['Status'] = 'OK'
                if state in self.CAMERA_STATUS:
                    if self.CAMERA_STATUS[state] == 'DISCONNECTED':
                        self.data['CONNECTION']['CONNECT'] = 'Off'
                    else:
                        self.data['CONNECTION']['CONNECT'] = 'On'
                else:
                    self.logger.error('Unknown camera status: {0}, message: {1}'.format(state, message))
            else:
                self.application['Status'] = 'ERROR'
                self.data['CONNECTION']['CONNECT'] = 'Off'
        else:
            self.application['Status'] = 'ERROR'
            self.data['CONNECTION']['CONNECT'] = 'Off'

    def getCameraProps(self):
        value = self.SgGetCameraProps()
        if value['Success']:
            if 'GainValues' not in value:
                self.data['Gain'] = ['High']
            else:
                self.data['Gain'] = value['GainValues'][0]
            self.data['CCD_INFO'] = {}
            self.data['CCD_INFO']['CCD_MAX_X'] = value['NumPixelsX']
            self.data['CCD_INFO']['CCD_MAX_Y'] = value['NumPixelsY']

    def getImage(self, imageParams):
        path = ''
        self.data['Imaging'] = True
        self.mutexCancel.lock()
        self.cancel = False
        self.mutexCancel.unlock()

        # waiting for start integrating
        self.main.cameraStatusText.emit('START')
        suc, mes, guid = self.SgCaptureImage(binningMode=imageParams['Binning'],
                                             exposureLength=imageParams['Exposure'],
                                             iso=str(imageParams['Iso']),
                                             gain=imageParams['Gain'],
                                             speed=imageParams['Speed'],
                                             frameType='Light',
                                             filename=imageParams['File'],
                                             path=imageParams['BaseDirImages'],
                                             useSubframe=imageParams['CanSubframe'],
                                             posX=imageParams['OffX'],
                                             posY=imageParams['OffY'],
                                             width=imageParams['SizeX'],
                                             height=imageParams['SizeY'])
        self.logger.info('SgCaptureImage: {0}'.format(mes))
        if not suc:
            self.logger.warning('Imaging no start, message: {0}'.format(mes))
            self.main.cameraStatusText.emit('ERROR')
            imageParams['Imagepath'] = ''
            self.mutexCancel.lock()
            self.cancel = True
            self.mutexCancel.unlock()

        # loop for integrating
        self.main.cameraStatusText.emit('INTEGRATE')
        # wait for start integrating
        while not self.cancel:
            suc, state, message = self.SgGetDeviceStatus('Camera')
            if 'integrating' in message:
                break
            time.sleep(0.1)
        # bow for the duration
        while not self.cancel:
            suc, state, message = self.SgGetDeviceStatus('Camera')
            if 'downloading' in message or 'ready' in message or 'idle' in message:
                break
            time.sleep(0.1)

        # Loop for downloading
        self.main.imageIntegrated.emit()
        self.main.cameraStatusText.emit('DOWNLOAD')
        while not self.cancel:
            suc, path = self.SgGetImagePath(guid)
            if suc:
                break
            time.sleep(0.1)

        # Loop for saving
        self.main.imageDownloaded.emit()
        self.main.cameraStatusText.emit('SAVING')
        while not self.cancel:
            suc, state, message = self.SgGetDeviceStatus('Camera')
            if 'ready' in message or 'idle' in message:
                break
            time.sleep(0.1)

        # finally idle
        self.main.cameraStatusText.emit('IDLE')
        self.main.cameraExposureTime.emit('')
        imageParams['Imagepath'] = path.replace('\\', '/')

    def SgCaptureImage(self, binningMode=1, exposureLength=1,
                       gain=None, iso=None, speed=None, frameType=None, filename=None,
                       path=None, useSubframe=False, posX=0, posY=0,
                       width=1, height=1):
        # reference {"BinningMode":0,"ExposureLength":0,"Gain":"String","Speed":"Normal","FrameType":"Light",
        # reference "Path":"String","UseSubframe":false,"X":0,"Y":0,"Width":0,"Height":0}
        data = {"BinningMode": binningMode, "ExposureLength": exposureLength, "UseSubframe": useSubframe, "X": posX, "Y": posY,
                "Width": width, "Height": height}
        if gain:
            data['Gain'] = gain
        if iso:
            data['Iso'] = iso
        if speed:
            data['Speed'] = speed
        if frameType:
            data['FrameType'] = frameType
        if path and filename:
            data['Path'] = path + '/' + filename
        try:
            result = requests.post(self.ipSGPro + self.captureImagePath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            return result['Success'], result['Message'], result['Receipt']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', ''

    def SgGetCameraProps(self):
        # reference {}
        data = {}
        try:
            result = requests.post(self.ipSGPro + self.getCameraPropsPath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            return result
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', '', '', ''

    def SgGetDeviceStatus(self, device):
        # reference {"Device": "Camera"}, devices are "Camera", "FilterWheel", "Focuser", "Telescope" and "PlateSolver"}
        data = {'Device': device}
        try:
            result = requests.post(self.ipSGPro + self.getDeviceStatusPath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            if 'Message' not in result:
                result['Message'] = 'None'
            return result['Success'], result['State'], result['Message']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed', 'Request failed'

    def SgGetImagePath(self, _guid):
        # reference {"Receipt":"00000000000000000000000000000000"}
        data = {'Receipt': _guid}
        try:
            result = requests.post(self.ipSGPro + self.getImagePath, data=bytes(json.dumps(data).encode('utf-8')))
            result = json.loads(result.text)
            return result['Success'], result['Message']
        except Exception as e:
            self.logger.error('error: {0}'.format(e))
            return False, 'Request failed'



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import PyQt5


class MWStyles:

    COLOR_ASTRO = PyQt5.QtGui.QColor(32, 144, 192)
    COLOR_BLUE = PyQt5.QtGui.QColor(0, 0, 255)
    COLOR_YELLOW = PyQt5.QtGui.QColor(192, 192, 0)
    COLOR_GREEN = PyQt5.QtGui.QColor(0, 255, 0)
    COLOR_WHITE = PyQt5.QtGui.QColor(255, 255, 255)
    COLOR_RED = PyQt5.QtGui.QColor(255, 0, 0)
    COLOR_ORANGE = PyQt5.QtGui.QColor(192, 96, 96)
    COLOR_BLACK = PyQt5.QtGui.QColor(0, 0, 0)

    # define the basic style of the mountwizzard3 theme
    # rise the font size for retina displays
    MAC_STYLE = """
    QWidget {
        background-color: #181818;
        font-family: Arial;
        font-style: normal;
        font-size: 13pt;
    }
    QWidget[large='true'] {
        font-size: 36pt;
    }
    QGroupBox{
        font-family: Arial;
        font-style: normal;
        font-size: 10pt;
    } 
    """
    NON_MAC_STYLE = """
    QWidget {
        background-color: #181818;
        font-family: Arial;
        font-style: normal;
        font-size: 10pt;
    }
    QWidget[large='true'] {
        font-size: 28pt;
    } 
    QGroupBox{
        font-family: Arial;
        font-style: normal;
        font-size: 8pt;
    }     
    """
    BASIC_STYLE = """
    QToolTip {
        border-width: 2px;
        border-style: outset;
        border-color: #404040;
        background-color: #202020;
        color: #C0C0C0;
        padding: 5px;
    }
    QLabel {
        background-color: transparent;
        color: #C0C0C0;
    }
    QLabel[color='blue'] {
        border-width: 3px;
        border-color: rgb(16, 72, 96);
        border-style: outset;
        border-radius: 2px;
        background-color: rgb(9, 36, 48);
    }
    QLabel[iconpicture='true'] {
        border-width: 2px;
        border-color: #404040;
        border-style: outset;
        border-radius: 2px;
    }
    
    /* QLine Edit*/
    QLineEdit {
        background-color: #000000;
        color: rgb(32, 144, 192);
        text-align: right;
        border-width: 1px;
        border-color: #404040;
        border-style: outset;
        border-radius: 2px;
    }
    QLineEdit:disabled {
        border-width: 1px;
        border-color: #000000;
        border-style: flat;
        border-radius: 2px;
    }
    QLineEdit {
        border-width: 1px;
        border-color: red;
        border-style: flat;
        border-radius: 2px;
    }
    QLineEdit[check='false'] {
        background-color: #101010;
        color: rgb(255, 0, 0);
    }
    QLineEdit[check='true'] {
        background-color: #101010;
        color: rgb(32, 144, 192);
    }
    
    /* Checkboxes */
    QCheckBox {
        color: #C0C0C0;
        spacing: 8px;
        background-color: transparent;
    }
    QCheckBox::indicator {
        border-width: 1px;
        border-color: #404040;
        background-color: #101010;
        border-style: outset;
        border-radius: 2px;
        width: 13px;
        height: 13px;
    }
    QCheckBox::indicator:checked {
        background-color: rgb(32, 144, 192);
        image: url(:/checkmark.ico);
    }

    /* Group Box */
    QGroupBox {
        background-color: #181818;
        border-width: 1px;
        border-style: outset;
        border-radius: 3px;
        border-color: #404040;
        margin-top: 6px;
    }
    QGroupBox::title {
        left: 5px;
        subcontrol-origin: margin;
        subcontrol-position: top left; /* position at the top center */
        color: #C0C0C0;
        background-color: #181818;
    }
    QRadioButton {
        color: #C0C0C0;
        background-color: transparent;
    }
    QRadioButton:disabled {
        color: #404040;
        background-color: transparent;
    }
    QRadioButton::indicator {
        border-width: 1px;
        border-color: #404040;
        background-color: #101010;
        border-style: outset;
        border-radius: 2px;
        width: 13px;
        height: 13px;
    }
    QRadioButton::indicator:checked {
        background-color: rgb(32, 144, 192);
        image: url(:/checkmark.ico);
    }
    
    /* Spin Boxes */
    QDoubleSpinBox {
        background-color: #101010;
        color: rgb(32, 144, 192);
        text-align: right;

        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        padding-right: 2px;
    }
    QDoubleSpinBox::up-button {
        subcontrol-origin: border;
        subcontrol-position: top right; /* position at the top right corner */
        width: 12px; /* 16 + 2*1px border-width = 15px padding + 3px parent border */
        border-width: 1px;
        border-radius: 2px;
        border-color: #404040;
        border-style: outset;
        background-color: #181818;
    }
    QDoubleSpinBox::up-arrow {
        image: url(:/arrow-up.ico);
        width: 12px;
        height: 16px;
    }
    QDoubleSpinBox::down-button {
        subcontrol-origin: border;
        subcontrol-position: bottom right; /* position at the top right corner */
        width: 12px; /* 16 + 2*1px border-width = 15px padding + 3px parent border */
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        border-color: #404040;
        background-color: #181818;
    }
    QDoubleSpinBox::down-arrow {
        image: url(:/arrow-down.ico);
        width: 12px;
        height: 16px;
    }
    
    /* Push Buttons */
    QPushButton {
        background-color: #202020;
        color: #C0C0C0;
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        min - width: 10em;
    }

    QInputDialog QPushButton {
        background-color: #202020;
        color: #C0C0C0;
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        min-width: 90px;
        min-height: 25px;
    }
    QInputDialog QPushButton:default {
        border-width: 2px;
        border-color: rgb(32, 144, 192);
    }

    QMessageBox QPushButton {
        background-color: #202020;
        color: #C0C0C0;
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        min-width: 90px;
        min-height: 25px;
    }
    QMessageBox QPushButton:default {
        border-width: 2px;
        border-color: rgb(32, 144, 192);
    }

    QListWidget {
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        color: #C0C0C0;
        padding-left: 5px;
        padding-top: 5px;
    }

    QPushButton:pressed {
        border-color: #404040;
        border-width: 2px;
        border-style: inset;
        border-radius: 2px;
    }
    QPushButton[running='false'] {
        background-color: #202020;
        color: #C0C0C0;
    }     
    QPushButton[running='true'] {
        background-color: rgb(32, 144, 192);
        color: #000000;
    } 
    QPushButton[cancel='true'] {
        background-color: rgb(192,0, 0);
        color: #C0C0C0;
    } 
    QPushButton[cancel='false'] {
        background-color: #202020;
        color: #C0C0C0;
    }
    QPushButton[iconset='true'] {
        text-align: center;
        padding-left: 3px;
        padding-right: 3px;
    }
    QPushButton:focus {
        outline: none;
    }
    QPushButton:disabled {
        background-color: #101010;
        color: #202020;
        border-color: #202020;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
    }
    QPushButton[color='gray'] {
        background-color: gray;
        color: #000000;
    }
    QPushButton[color='red'] {
        background-color: red;
        color: #000000;
    }
    QPushButton[color='yellow'] {
        background-color: yellow;
        color: #000000;
    }
    QPushButton[color='green'] {
        background-color: green;
        color: #000000;
    }
    /* Combo Boxes */
    QComboBox {
        combobox-popup: 0;
        color: #C0C0C0;
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        padding-left: 5px;
        background-color: #202020;
    }
    QComboBox::drop-down {
        subcontrol-origin: border;
        subcontrol-position: right; /* position at the top right corner */
        width: 20px; /* 16 + 2*1px border-width = 15px padding + 3px parent border */
        border-color: #404040;
        border-width: 1px;
        border-style: outset;
        border-radius: 2px;
        background-color: #202020;
    }
    QComboBox::down-arrow {
        image: url(:/arrow-down.ico);
        width: 20px;
        height: 31px;
    }
     QComboBox QListView {
        border-width: 2px;
        border-style: outset;
        border-color: #404040;
        border-radius: 2px;
        color: #C0C0C0;
        background-color: #101010;
        min-height: 60px;
    }
    QComboBox QListView::item {
        min-height: 28px;
    }
    QComboBox QListView::item:selected { 
        border-width: 1px;
        border-style: outset;
        border-color: #404040;
        border-radius: 2px; 
        color: #101010;
        background-color: rgb(32, 144, 192);
    }
    QComboBox QListView::item:!selected { 
    }

    /* lines */
    QFrame[frameShape="4"] {/* horizontal lines */
        color: rgb(16, 72, 96);
    }
    QFrame[frameShape="5"] {/* vertical lines */
        color: rgb(16, 72, 96);
    }
    
    /* tab widget */
    QTabWidget:pane {
        border-width: 2px;
        border-color: #404040;
        border-radius: 2px;
        border-style: outset;
    }
    /* needed for MAC OSX */
    QTabWidget:tab-bar {
        alignment: center;
    }
    QTabBar::tab {
        background-color: #202020;
        color: #C0C0C0;
        border-width: 2px;
        border-color: rgb(16, 72, 96);
        border-radius: 2px;
        border-style: outset;
        padding: 4px;
        padding-left: 4px;
        padding-right: 4px;
    }
    QTabBar::tab:selected {
        color: #000000;
        background: rgb(32, 144, 192);
    }
    QTabBar::tab:!selected {
        margin-top: 4px;
    }
    QTabBar::tab:only-one {
        margin: 1;
    }
    QTabBar::tab:!enabled {
        color: #404040;
        border-color: #202020;
    }
    
    /* scroll bar */
    QScrollBar:vertical
    {   background-color: #202020;
        width: 20px;
        margin: 20px 3px 20px 3px;
        border-width: 1px;
        border-color: #404040;
        border-style: outset;
        border-radius: 3px;
    }
    QScrollBar::handle:vertical
    {   background-color:  rgb(32, 144, 192);
        min-height: 10px;
        border-radius: 3px;
    }
    QScrollBar::sub-line:vertical
    {   margin: 3px 0px 3px 0px;
        border-image: url(:arrow-up.ico);
        height: 16px;
        width: 16px;
        subcontrol-position: top;
        subcontrol-origin: margin;
    }
    QScrollBar::add-line:vertical
    {   margin: 3px 0px 3px 0px;
        border-image: url(:arrow-down.ico);
        height: 16px;
        width: 16px;
        subcontrol-position: bottom;
        subcontrol-origin: margin;
    }
    QScrollBar::sub-line:vertical:on
    {   border-image: url(:arrow-up.png);
        height: 16px;
        width: 16px;
        subcontrol-position: top;
        subcontrol-origin: margin;
    }
    QScrollBar::add-line:vertical:on
    {   border-image: url(:arrow-down.png);
        height: 16px;
        width: 16px;
        subcontrol-position: bottom;
        subcontrol-origin: margin;
    }
    QScrollBar::up-arrow:vertical, QScrollBar::down-arrow:vertical
    {   background: none;
        border-width: 1px;
        border-color: #404040;
        border-style: outset;
        border-radius: 4px;
    }
    QScrollBar::add-page:vertical, QScrollBar::sub-page:vertical
    {   background: none;
    }    

    /* progress bar */
    QProgressBar {
        background-color: #101010;
        border-radius: 3px;
        border-width: 1px;
        border-color: #404040;
        border-style: outset;
    }
    QProgressBar::chunk {
        background-color: rgb(32, 144, 192);
        width: 1px;
        margin: 0px;
        border-width: 0px; 
        border-color: #404040;
        border-radius: 0px;
        border-style: outset;
    }
    QTextBrowser {
        background-color: #101010;
        border-radius: 3px;
        border-width: 2px;
        border-color: #404040;
        border-style: outset;
        margin: -5px;
    }
    """


############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import os
import logging
import time
import PyQt5
import requests
import queue
import comtypes.client
from pywinauto import Application, timings, findwindows, application
from pywinauto.controls.win32_controls import ButtonWrapper, EditWrapper


class Automation(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    CYCLE = 500
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    UTC_1 = 'http://maia.usno.navy.mil/ser7/finals.data'
    UTC_2 = 'http://maia.usno.navy.mil/ser7/tai-utc.dat'
    COMETS = 'http://www.minorplanetcenter.net/iau/MPCORB/CometEls.txt'
    COMETS_START = 102
    COMETS_END = 161
    ASTEROIDS_START = 165
    ASTEROIDS_END = 196
    ASTEROIDS_MPC5000 = 'http://www.ap-i.net/pub/skychart/mpc/mpc5000.dat'
    # ASTEROIDS = 'http://www.minorplanetcenter.net/iau/MPCORB/MPCORB.DAT'
    ASTEROIDS_NEA = 'http://www.minorplanetcenter.net/iau/MPCORB/NEA.txt'
    ASTEROIDS_PHA = 'http://www.minorplanetcenter.net/iau/MPCORB/PHA.txt'
    ASTEROIDS_TNO = 'http://www.minorplanetcenter.net/iau/MPCORB/Distant.txt'
    ASTEROIDS_UNUSAL = 'http://www.minorplanetcenter.net/iau/MPCORB/Unusual.txt'
    SPACESTATIONS = 'http://www.celestrak.com/NORAD/elements/stations.txt'
    SATBRIGHTEST = 'http://www.celestrak.com/NORAD/elements/visual.txt'
    COMETS_FILE = 'comets.mpc'
    ASTEROIDS_FILE = 'asteroids.mpc'
    SPACESTATIONS_FILE = 'spacestations.tle'
    SATBRIGHTEST_FILE = 'satbrightest.tle'
    UTC_1_FILE = 'finals.data'
    UTC_2_FILE = 'tai-utc.dat'
    OPENDIALOG = 'Dialog'

    def __init__(self, app, thread):
        super().__init__()
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()

        self.app = app
        self.thread = thread
        self.commandDispatcherQueue = queue.Queue()
        self.cycleTimer = None

        self.appAvailable = False
        self.appName = ''
        self.appInstallPath = ''
        self.appExe = 'GmQCIv2.exe'
        # defining the command dispatcher
        self.commandDispatch = {
            'COMETS':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadComets,
                            'Method': self.downloadFile,
                            'Parameter': ['self.COMETS',
                                          'self.TARGET_DIR + self.COMETS_FILE'],
                            'Checkbox': self.app.ui.checkComets
                        }
                    ]
                },
            'UPLOADMOUNT':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_uploadMount,
                            'Method': self.uploadMount
                        }
                    ]
                },
            'SPACESTATIONS':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadSpacestations,
                            'Method': self.downloadFile,
                            'Parameter': ['self.SPACESTATIONS',
                                          'self.TARGET_DIR + self.SPACESTATIONS_FILE'],
                            'Checkbox': self.app.ui.checkTLE
                        }
                    ]
                },
            'SATBRIGHTEST':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadSatbrighest,
                            'Method': self.downloadFile,
                            'Parameter': ['self.SATBRIGHTEST',
                                          'self.TARGET_DIR + self.SATBRIGHTEST_FILE'],
                            'Checkbox': self.app.ui.checkTLE
                        }
                    ]
                },
            'ASTEROIDS_MPC5000':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadAsteroidsMPC5000,
                            'Method': self.downloadFile,
                            'Parameter': ['self.ASTEROIDS_MPC5000',
                                          'self.TARGET_DIR + self.ASTEROIDS_FILE'],
                            'Checkbox': self.app.ui.checkAsteroids
                        }
                    ]
                },
            'ASTEROIDS_NEA':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadAsteroidsNEA,
                            'Method': self.downloadFile,
                            'Parameter': ['self.ASTEROIDS_NEA',
                                          'self.TARGET_DIR + self.ASTEROIDS_FILE'],
                            'Checkbox': self.app.ui.checkAsteroids
                        }
                    ]
                },
            'ASTEROIDS_PHA':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadAsteroidsPHA,
                            'Method': self.downloadFile,
                            'Parameter': ['self.ASTEROIDS_PHA',
                                          'self.TARGET_DIR + self.ASTEROIDS_FILE'],
                            'Checkbox': self.app.ui.checkAsteroids
                        }
                    ]
                },
            'ASTEROIDS_TNO':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadAsteroidsTNO,
                            'Method': self.downloadFile,
                            'Parameter': ['self.ASTEROIDS_TNO',
                                          'self.TARGET_DIR + self.ASTEROIDS_FILE'],
                            'Checkbox': self.app.ui.checkAsteroids
                        }]
                },
            'EARTHROTATION':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadEarthrotation,
                            'Method': self.downloadFile,
                            'Parameter': ['self.UTC_1',
                                          'self.TARGET_DIR + self.UTC_1_FILE']
                        },
                        {
                            'Button': self.app.ui.btn_downloadEarthrotation,
                            'Method': self.downloadFile,
                            'Parameter': ['self.UTC_2',
                                          'self.TARGET_DIR + self.UTC_2_FILE'],
                            'Checkbox': self.app.ui.checkEarthrotation
                        }
                    ]
                },
            'ALL':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_downloadEarthrotation,
                            'Method': self.downloadFile,
                            'Parameter': ['self.UTC_1',
                                          'self.TARGET_DIR + self.UTC_1_FILE']
                        },
                        {
                            'Button': self.app.ui.btn_downloadEarthrotation,
                            'Method': self.downloadFile,
                            'Parameter': ['self.UTC_2',
                                          'self.TARGET_DIR + self.UTC_2_FILE'],
                            'Checkbox': self.app.ui.checkEarthrotation
                        },
                        {
                            'Button': self.app.ui.btn_downloadSpacestations,
                            'Method': self.downloadFile,
                            'Parameter': ['self.SPACESTATIONS',
                                          'self.TARGET_DIR + self.SPACESTATIONS_FILE'],
                            'Checkbox': self.app.ui.checkTLE
                        },
                        {
                            'Button': self.app.ui.btn_downloadSatbrighest,
                            'Method': self.downloadFile,
                            'Parameter': ['self.SATBRIGHTEST',
                                          'self.TARGET_DIR + self.SATBRIGHTEST_FILE'],
                            'Checkbox': self.app.ui.checkTLE
                        },
                        {
                            'Button': self.app.ui.btn_downloadComets,
                            'Method': self.downloadFile,
                            'Parameter': ['self.COMETS',
                                          'self.TARGET_DIR + self.COMETS_FILE'],
                            'Checkbox': self.app.ui.checkComets
                        },
                        {
                            'Button': self.app.ui.btn_downloadAsteroidsMPC5000,
                            'Method': self.downloadFile,
                            'Parameter': ['self.ASTEROIDS_MPC5000',
                                          'self.TARGET_DIR + self.ASTEROIDS_FILE'],
                            'Checkbox': self.app.ui.checkAsteroids
                        }
                    ]
                }
            }

        self.checkApplication()
        self.TARGET_DIR = self.appInstallPath
        if self.TARGET_DIR == '':
            self.TARGET_DIR = os.getcwd()+'/config/'
        # signal slot
        self.app.ui.btn_downloadEarthrotation.clicked.connect(lambda: self.commandDispatcherQueue.put('EARTHROTATION'))
        self.app.ui.btn_downloadSpacestations.clicked.connect(lambda: self.commandDispatcherQueue.put('SPACESTATIONS'))
        self.app.ui.btn_downloadSatbrighest.clicked.connect(lambda: self.commandDispatcherQueue.put('SATBRIGHTEST'))
        self.app.ui.btn_downloadAsteroidsMPC5000.clicked.connect(lambda: self.commandDispatcherQueue.put('ASTEROIDS_MPC5000'))
        self.app.ui.btn_downloadAsteroidsNEA.clicked.connect(lambda: self.commandDispatcherQueue.put('ASTEROIDS_NEA'))
        self.app.ui.btn_downloadAsteroidsPHA.clicked.connect(lambda: self.commandDispatcherQueue.put('ASTEROIDS_PHA'))
        self.app.ui.btn_downloadAsteroidsTNO.clicked.connect(lambda: self.commandDispatcherQueue.put('ASTEROIDS_TNO'))
        self.app.ui.btn_downloadComets.clicked.connect(lambda: self.commandDispatcherQueue.put('COMETS'))
        self.app.ui.btn_downloadAll.clicked.connect(lambda: self.commandDispatcherQueue.put('ALL'))
        self.app.ui.btn_uploadMount.clicked.connect(lambda: self.commandDispatcherQueue.put('UPLOADMOUNT'))

    def initConfig(self):
        try:
            if 'FilterExpressionMPC' in self.app.config:
                self.app.ui.le_filterExpressionMPC.setText(self.app.config['FilterExpressionMPC'])
            if 'CheckFilterMPC' in self.app.config:
                self.app.ui.checkFilterMPC.setChecked(self.app.config['CheckFilterMPC'])
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass

    def storeConfig(self):
        self.app.config['CheckFilterMPC'] = self.app.ui.checkFilterMPC.isChecked()
        self.app.config['FilterExpressionMPC'] = self.app.ui.le_filterExpressionMPC.text()

    def checkApplication(self):
        self.appAvailable, self.appName, self.appInstallPath = self.app.checkRegistrationKeys('10micron QCI')
        if self.appAvailable:
            self.app.messageQueue.put('Found: {0}\n'.format(self.appName))
            self.logger.info('Name: {0}, Path: {1}'.format(self.appName, self.appInstallPath))
        else:
            self.logger.info('Application 10micron Updater  not found on computer')

    def run(self):
        self.logger.info('automation started')
        # a running thread is shown with variable isRunning = True. This thread should hav it's own event loop.
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('automation stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.commandDispatcherQueue.empty():
            command = self.commandDispatcherQueue.get()
            self.commandDispatcher(command)

    def commandDispatcher(self, command):
        # if we have a command in dispatcher
        if command in self.commandDispatch:
            # running through all necessary commands
            for work in self.commandDispatch[command]['Worker']:
                # if we want to color a button, which one
                if 'Button' in work:
                    work['Button'].setProperty('running', True)
                    work['Button'].style().unpolish(work['Button'])
                    work['Button'].style().polish(work['Button'])
                if 'Parameter' in work:
                    parameter = []
                    for p in work['Parameter']:
                        parameter.append(eval(p))
                    work['Method'](*parameter)
                else:
                    work['Method']()
                if 'Checkbox' in work:
                    work['Checkbox'].setChecked(True)
                if 'Button' in work:
                    work['Button'].setProperty('running', False)
                    work['Button'].style().unpolish(work['Button'])
                    work['Button'].style().polish(work['Button'])

    def filterFileMPC(self, directory, filename, expression, start, end):
        numberEntry = 0
        with open(directory + filename, 'r') as inFile, open(directory + 'filter.mpc', 'w') as outFile:
            for line in inFile:
                searchExp = expression.split(',')
                for exp in searchExp:
                    if line.find(exp, start, end) != -1:
                        outFile.write(line)
                        numberEntry += 1
        if numberEntry == 0:
            return False
        else:
            self.app.messageQueue.put('Found {0} target(s) in MPC file: {1}\n'.format(numberEntry, filename))
            self.logger.info('Found {0} target(s) in MPC file: {1}!'.format(numberEntry, filename))
            return True

    def downloadFile(self, url, filename):
        try:
            r = requests.get(url, stream=True)
            if r.status_code == 200:
                numberOfChunks = 0
                with open(filename, 'wb') as f:
                    for chunk in r.iter_content(128):
                        numberOfChunks += 1
                        f.write(chunk)
            self.app.messageQueue.put('Downloaded {0} Bytes\n'.format(128 * numberOfChunks))
        except Exception as e:
            self.logger.error('Download of {0} failed, error{1}'.format(url, e))
            self.app.messageQueue.put('#BRDownload Error {0}\n'.format(e))
        return

    def uploadMount(self):
        actual_work_dir = ''
        try:
            actual_work_dir = os.getcwd()
            os.chdir(os.path.dirname(self.appInstallPath))
            app = Application(backend='win32')
            app.start(self.appInstallPath + self.appExe)
            # timings.Timings.Slow()
        except application.AppStartError:
            self.logger.error('Failed to start updater, please check!')
            self.app.messageQueue.put('#BRFailed to start updater, please check\n')
            os.chdir(actual_work_dir)
            return
        try:
            dialog = timings.WaitUntilPasses(2, 0.2, lambda: findwindows.find_windows(title='GmQCIv2', class_name='#32770')[0])
            winOK = app.window_(handle=dialog)
            winOK['OK'].click()
        except timings.TimeoutError as e:
            self.logger.warning('No invalid floating point windows occurred - moving forward')
        except Exception as e:
            self.logger.error('error{0}'.format(e))
        finally:
            pass
        try:
            win = app['10 micron control box update']                                                                       # link handle
            win['next'].click()                                                                                             # accept next
            win['next'].click()                                                                                             # go upload select page
            ButtonWrapper(win['Control box firmware']).UncheckByClick()                                                       # no firmware updates
        except Exception as e:
            self.logger.error('error{0}'.format(e))
            self.app.messageQueue.put('#BRError in starting 10micron updater, please check\n')
            os.chdir(actual_work_dir)
            return
        ButtonWrapper(win['Orbital parameters of comets']).UncheckByClick()
        ButtonWrapper(win['Orbital parameters of asteroids']).UncheckByClick()
        ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
        ButtonWrapper(win['UTC / Earth rotation data']).UncheckByClick()
        try:
            uploadNecessary = False
            if self.app.ui.checkComets.isChecked():
                ButtonWrapper(win['Orbital parameters of comets']).CheckByClick()
                win['Edit...4'].click()
                popup = app['Comet orbits']
                popup['MPC file'].click()
                filedialog = app[self.OPENDIALOG]
                if self.app.ui.checkFilterMPC.isChecked():
                    if self.filterFileMPC(self.TARGET_DIR, self.COMETS_FILE, self.app.ui.le_filterExpressionMPC.text(), self.COMETS_START, self.COMETS_END):
                        uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + 'filter.mpc')                               # filename box
                else:
                    uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.COMETS_FILE)                           # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
            else:
                ButtonWrapper(win['Orbital parameters of comets']).UncheckByClick()
            if self.app.ui.checkAsteroids.isChecked():
                ButtonWrapper(win['Orbital parameters of asteroids']).CheckByClick()
                win['Edit...3'].click()
                popup = app['Asteroid orbits']
                popup['MPC file'].click()
                filedialog = app[self.OPENDIALOG]
                if self.app.ui.checkFilterMPC.isChecked():
                    if self.filterFileMPC(self.TARGET_DIR, self.ASTEROIDS_FILE, self.app.ui.le_filterExpressionMPC.text(), self.ASTEROIDS_START, self.ASTEROIDS_END):
                        uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + 'filter.mpc')
                else:
                    uploadNecessary = True
                    EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.ASTEROIDS_FILE)                        # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
            else:
                ButtonWrapper(win['Orbital parameters of asteroids']).UncheckByClick()
            if self.app.ui.checkTLE.isChecked():
                ButtonWrapper(win['Orbital parameters of satellites']).CheckByClick()
                win['Edit...2'].click()
                popup = app['Satellites orbits']
                popup['Load from file'].click()
                filedialog = app[self.OPENDIALOG]
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.SATBRIGHTEST_FILE)                         # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
            if self.app.ui.checkTLE.isChecked():
                ButtonWrapper(win['Orbital parameters of satellites']).CheckByClick()
                win['Edit...2'].click()
                popup = app['Satellites orbits']
                popup['Load from file'].click()
                filedialog = app[self.OPENDIALOG]
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.SPACESTATIONS_FILE)                        # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                popup['Close'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['Orbital parameters of satellites']).UncheckByClick()
            if self.app.ui.checkEarthrotation.isChecked():
                ButtonWrapper(win['UTC / Earth rotation data']).CheckByClick()
                win['Edit...1'].click()
                popup = app['UTC / Earth rotation data']
                popup['Import files...'].click()
                filedialog = app['Open finals data']
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.UTC_1_FILE)                                # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                filedialog = app['Open tai-utc.dat']
                EditWrapper(filedialog['Edit13']).SetText(self.TARGET_DIR + self.UTC_2_FILE)                                # filename box
                filedialog['Button16'].click()                                                                              # accept filename selection and proceed
                fileOK = app['UTC data']
                fileOK['OK'].click()
                uploadNecessary = True
            else:
                ButtonWrapper(win['UTC / Earth rotation data']).UncheckByClick()
        except Exception as e:
            self.logger.error('error{0}'.format(e))
            self.app.messageQueue.put('#BRError in choosing upload files, please check 10micron updater\n')
            os.chdir(actual_work_dir)
            return
        if not self.app.workerMountDispatcher.mountStatus['Once']:
            self.app.messageQueue.put('Upload only possible with connected mount !')
            uploadNecessary = False
        if uploadNecessary:
            try:
                win['next'].click()
                win['next'].click()
                win['Update Now'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('#BRError in uploading files, please check 10micron updater\n')
                os.chdir(actual_work_dir)
                return
            try:
                dialog = timings.WaitUntilPasses(60, 0.5, lambda: findwindows.find_windows(title='Update completed', class_name='#32770')[0])
                winOK = app.window_(handle=dialog)
                winOK['OK'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('#BRError in closing 10micron updater, please check\n')
                os.chdir(actual_work_dir)
                return
        else:
            try:
                win['Cancel'].click()
                winOK = app['Exit updater']
                winOK['Yes'].click()
            except Exception as e:
                self.logger.error('error{0}'.format(e))
                self.app.messageQueue.put('#BRError in closing Updater, please check\n')
                os.chdir(actual_work_dir)
                return


if __name__ == "__main__":
    pass



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import math
import datetime
import PyQt5
from astropy import _erfa


class Transform:
    logger = logging.getLogger(__name__)

    def __init__(self, app):
        self.app = app
        self.ERFA = _erfa
        self.mutexERFA = PyQt5.QtCore.QMutex()
        self.mutexTopocentric = PyQt5.QtCore.QMutex()
        # if nothing is present, use the coordinates of greenwich
        self.siteLat = 51.476852
        self.siteLon = 0
        self.siteHeight = 46
        # date of 01.05.2018
        self.julianDate = 2458240
        # connect data transfer
        self.app.signalMountSiteData.connect(self.setSiteData)
        self.app.signalJulianDate.connect(self.setJulianDate)

    def setSiteData(self, lat, lon, height):
        self.siteLat = self.degStringToDecimal(lat)
        self.siteLon = self.degStringToDecimal(lon)
        self.siteHeight = float(height)

    def setJulianDate(self, jd):
        self.julianDate = jd

    def topocentricToAzAlt(self, ra, dec):
        self.mutexTopocentric.lock()
        ra = (ra * 360 / 24 + 360.0) % 360.0
        dec = math.radians(dec)
        ra = math.radians(ra)
        lat = math.radians(self.siteLat)
        alt = math.asin(math.sin(dec) * math.sin(lat) + math.cos(dec) * math.cos(lat) * math.cos(ra))
        value = (math.sin(dec) - math.sin(alt) * math.sin(lat)) / (math.cos(alt) * math.cos(lat))
        # we have to check for rounding error, which could happen
        if value > 1:
            value = 1
        elif value < -1:
            value = -1
        A = math.acos(value)
        A = math.degrees(A)
        alt = math.degrees(alt)
        if math.sin(ra) >= 0.0:
            az = 360.0 - A
        else:
            az = A
        self.mutexTopocentric.unlock()
        return az, alt

    def degStringToDecimal(self, value, splitter=':'):
        returnValue = 0
        sign = 1
        if '-' in value:
            value = value.replace('-', '')
            sign = -1
        elif '+' in value:
            value = value.replace('+', '')
        else:
            # just for formal understanding
            pass
        try:
            if len(value.split(splitter)) == 3:
                hour, minute, second = value.split(splitter)
                returnValue = (float(hour) + float(minute) / 60 + float(second) / 3600) * sign
            elif len(value.split(splitter)) == 2:
                hour, minute = value.split(splitter)
                returnValue = (float(hour) + float(minute) / 60) * sign
        except Exception as e:
            self.logger.error('Error in conversion of:{0} with splitter:{1}, e:{2}'.format(value, splitter, e))
            returnValue = 0
        finally:
            pass
        return returnValue

    @staticmethod
    def decimalToDegree(value, with_sign=True, with_decimal=False, spl=':'):
        if value >= 0:
            sign = '+'
        else:
            sign = '-'
        value = abs(value)
        hour = int(value)
        minute = int((value - hour) * 60)
        second = int(((value - hour) * 60 - minute) * 60)
        if with_decimal:
            second_dec = '.{0:1d}'.format(int((((value - hour) * 60 - minute) * 60 - second) * 10))
        else:
            second_dec = ''
        if with_sign:
            returnValue = '{0}{1:02d}{5}{2:02d}{5}{3:02d}{4}'.format(sign, hour, minute, second, second_dec, spl)
        else:
            returnValue = '{0:02d}{4}{1:02d}{4}{2:02d}{3}'.format(hour, minute, second, second_dec, spl)
        return returnValue

    @staticmethod
    def decimalToDegreeMountSr(value):
        degree = int(value)
        minute = int((value - degree) * 60)
        second = int(((value - degree) * 60 - minute) * 60)
        second_dec = int((((value - degree) * 60 - minute) * 60 - second) * 100)
        returnValue = ':Sr{0:02d}:{1:02d}:{2:02d}.{3:02d}#'.format(degree, minute, second, second_dec)
        return returnValue

    @staticmethod
    def decimalToDegreeMountSd(value):
        if value >= 0:
            sign = '+'
        else:
            sign = '-'
        value = abs(value)
        degree = int(value)
        minute = int((value - degree) * 60)
        second = int(((value - degree) * 60 - minute) * 60)
        second_dec = int((((value - degree) * 60 - minute) * 60 - second) * 10)
        returnValue = ':Sd{0}{1:02d}*{2:02d}:{3:02d}.{4:01d}#'.format(sign, degree, minute, second, second_dec)
        return returnValue

    def transformERFA(self, ra, dec, transform=1):
        self.mutexERFA.lock()
        ts = datetime.datetime.utcnow()
        dut1_prev = self.ERFA.dat(ts.year, ts.month, ts.day, 0)
        dut1 = 37 + 4023.0 / 125.0 - dut1_prev
        # suc, tai1, tai2 = self.ERFA.eraUtctai(self.julianDate, 0)
        tai1, tai2 = self.ERFA.utctai(self.julianDate, 0)
        # tt1, tt2 = self.ERFA.eraTaitt(tai1, tai2)
        tt1, tt2 = self.ERFA.taitt(tai1, tai2)
        jdtt = tt1 + tt2
        date1 = self.julianDate
        date2 = 0

        if transform == 1:  # J2000 to Topo Az /Alt
            ra = ra % 24
            aob, zob, hob, dob, rob, eo = self.ERFA.atco13(ra * self.ERFA.D2PI / 24,
                                                           dec * self.ERFA.D2PI / 360,
                                                           0.0,
                                                           0.0,
                                                           0.0,
                                                           0.0,
                                                           date1 + date2,
                                                           0.0,
                                                           dut1,
                                                           self.siteLon * self.ERFA.DD2R,
                                                           self.siteLat * self.ERFA.DD2R,
                                                           self.siteHeight,
                                                           0.0,
                                                           0.0,
                                                           0.0,
                                                           0.0,
                                                           0.0,
                                                           0.0)
            val1 = aob * 360 / self.ERFA.D2PI
            val2 = 90.0 - zob * 360 / self.ERFA.D2PI

        elif transform == 2:                                                                                                # Topo to J2000
            rc, dc, eo = self.ERFA.atic13(self.ERFA.anp(ra * self.ERFA.D2PI / 24 + self.ERFA.eo06a(jdtt, 0.0)),
                                          dec * self.ERFA.D2PI / 360,
                                          date1 + date2,
                                          0.0)
            val1 = rc * 24.0 / self.ERFA.D2PI
            val2 = dc * self.ERFA.DR2D

        elif transform == 3:                                                                                                # J2000 to Topo
            ri, di, eo = self.ERFA.atci13(ra * self.ERFA.D2PI / 24,
                                          dec * self.ERFA.D2PI / 360,
                                          0,
                                          0,
                                          0,
                                          0,
                                          date1 + date2,
                                          0)
            val1 = self.ERFA.anp(ri - eo) * 24 / self.ERFA.D2PI
            val2 = di * 360 / self.ERFA.D2PI
        else:
            val1 = ra
            val2 = dec
        self.mutexERFA.unlock()
        return val1, val2



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import os
import platform
import PyQt5
import matplotlib
matplotlib.use('Qt5Agg')
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from baseclasses import styles

import time


class MwWidget(PyQt5.QtWidgets.QWidget, styles.MWStyles):

    logger = logging.getLogger(__name__)

    def __init__(self):
        super().__init__()
        self.palette = PyQt5.QtGui.QPalette()
        self.bundle_dir = ''
        self.showStatus = False
        self.initUI()
        self.screenSizeX = PyQt5.QtWidgets.QDesktopWidget().screenGeometry().width()
        self.screenSizeY = PyQt5.QtWidgets.QDesktopWidget().screenGeometry().height()

    def closeEvent(self, closeEvent):
        self.showStatus = False
        if self.windowTitle().startswith('MountWizzard'):
            self.quit()

    @staticmethod
    def widgetIcon(gui, icon):
        gui.setIcon(PyQt5.QtGui.QIcon(icon))
        gui.setProperty('iconset', True)
        gui.style().unpolish(gui)
        gui.style().polish(gui)
        gui.setIconSize(PyQt5.QtCore.QSize(16, 16))

    def initUI(self):
        self.setWindowFlags((self.windowFlags() | PyQt5.QtCore.Qt.CustomizeWindowHint) & ~PyQt5.QtCore.Qt.WindowMaximizeButtonHint)
        self.setMouseTracking(True)
        # sizing in gui should be fixed, because I have a static layout
        self.setFixedSize(790, 640)
        self.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        if platform.system() == 'Darwin':
            self.setStyleSheet(self.MAC_STYLE + self.BASIC_STYLE)
        else:
            self.setStyleSheet(self.NON_MAC_STYLE + self.BASIC_STYLE)

    @staticmethod
    def timeStamp():
        return time.strftime('%H:%M:%S - ', time.localtime())

    @staticmethod
    def changeStylesheet(ui, item, value):
        ui.setProperty(item, value)
        ui.style().unpolish(ui)
        ui.style().polish(ui)

    @staticmethod
    def selectFile(window, title, folder, filterSet, openFile=True):
        dlg = PyQt5.QtWidgets.QFileDialog()
        dlg.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        dlg.setStyleSheet('background-color: rgb(32,32,32); color: rgb(192,192,192)')
        dlg.setViewMode(PyQt5.QtWidgets.QFileDialog.List)
        dlg.setFileMode(PyQt5.QtWidgets.QFileDialog.ExistingFile)
        dlg.setNameFilter(filterSet)
        dlg.setModal(True)
        ph = window.geometry().height()
        px = window.geometry().x()
        py = window.geometry().y()
        dw = window.width()
        dh = window.height()
        dlg.setGeometry(px, py + ph - dh, dw, dh)
        ext = ''
        if openFile:
            value = dlg.getOpenFileName(dlg, title, os.getcwd() + folder, filterSet, options=PyQt5.QtWidgets.QFileDialog.DontUseNativeDialog)
        else:
            value = dlg.getSaveFileName(dlg, title, os.getcwd() + folder, filterSet, options=PyQt5.QtWidgets.QFileDialog.DontUseNativeDialog)
        name = value[0]
        if len(name) > 0:
            name, ext = os.path.splitext(name)
        return name, ext

    @staticmethod
    def selectDir(window, title, folder):
        dlg = PyQt5.QtWidgets.QFileDialog()
        dlg.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        dlg.setStyleSheet('background-color: rgb(32,32,32); color: rgb(192,192,192)')
        dlg.setViewMode(PyQt5.QtWidgets.QFileDialog.List)
        dlg.setModal(True)
        ph = window.geometry().height()
        px = window.geometry().x()
        py = window.geometry().y()
        dw = window.width()
        dh = window.height()
        dlg.setGeometry(px, py + ph - dh, dw, dh)
        value = dlg.getExistingDirectory(dlg, title, folder, options=PyQt5.QtWidgets.QFileDialog.DontUseNativeDialog)
        return value

    def dialogMessageLoadSaveDelete(self, window, title, question):
        dlg = PyQt5.QtWidgets.QMessageBox()
        if platform.system() == 'Darwin':
            dlg.setStyleSheet(self.MAC_STYLE + self.BASIC_STYLE)
        else:
            dlg.setStyleSheet(self.NON_MAC_STYLE + self.BASIC_STYLE)
        dlg.addButton('Load', PyQt5.QtWidgets.QMessageBox.YesRole)
        dlg.addButton('Save', PyQt5.QtWidgets.QMessageBox.YesRole)
        dlg.addButton('Delete', PyQt5.QtWidgets.QMessageBox.YesRole)
        cancel = dlg.addButton('Cancel', PyQt5.QtWidgets.QMessageBox.YesRole)
        dlg.setDefaultButton(cancel)
        dlg.setWindowTitle(title)
        dlg.setText(question)
        dlg.setIcon(PyQt5.QtWidgets.QMessageBox.Warning)
        dlg.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        px = window.geometry().x()
        py = window.geometry().y()
        dw = window.width()
        dh = window.height()
        dx = dlg.width()
        dy = dlg.height()
        dlg.setGeometry(px + (dw - dx), py + (dh - dy), dw, dh)
        return dlg.exec()

    @staticmethod
    def dialogMessage(window, title, question):
        dlg = PyQt5.QtWidgets.QMessageBox()
        dlg.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        dlg.setStyleSheet('background-color: rgb(32,32,32); color: rgb(192,192,192)')
        ph = window.geometry().height()
        px = window.geometry().x()
        py = window.geometry().y()
        dw = window.width()
        dh = window.height()
        dlg.setGeometry(px, py + ph - dh, dw, dh)
        return dlg.question(window, title, question, PyQt5.QtWidgets.QMessageBox.Ok | PyQt5.QtWidgets.QMessageBox.Cancel, PyQt5.QtWidgets.QMessageBox.Cancel)

    @staticmethod
    def dialogInputText(window, title, text, preset):
        dlg = PyQt5.QtWidgets.QInputDialog()
        dlg.setWindowIcon(PyQt5.QtGui.QIcon(':/mw.ico'))
        dlg.setStyleSheet('background-color: rgb(32,32,32); color: rgb(192,192,192)')
        ph = window.geometry().height()
        px = window.geometry().x()
        py = window.geometry().y()
        dw = window.width()
        dh = window.height()
        dlg.setGeometry(px, py + ph - dh, dw, dh)
        return dlg.getText(window, title, text, PyQt5.QtWidgets.QLineEdit.Normal, preset)


# class for embed the matplotlib in pyqt5 framework
class IntegrateMatplotlib(FigureCanvasQTAgg):

    def __init__(self, parent=None):
        helper = PyQt5.QtWidgets.QVBoxLayout(parent)
        self.fig = matplotlib.figure.Figure(dpi=75, facecolor=(25 / 256, 25 / 256, 25 / 256))
        FigureCanvasQTAgg.__init__(self, self.fig)
        helper.setContentsMargins(0, 0, 0, 0)
        self.setParent(parent)
        FigureCanvasQTAgg.updateGeometry(self)
        helper.addWidget(self)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import time
import PyQt5
import queue
from modeling import model_build


class ModelingDispatcher(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    signalModelPointsRedraw = PyQt5.QtCore.pyqtSignal()
    signalCancel = PyQt5.QtCore.pyqtSignal()

    CYCLE = 200
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    def __init__(self, app, thread):
        super().__init__()
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        # make main sources available
        self.app = app
        self.thread = thread
        self.cycleTimer = None
        self.commandDispatcherQueue = queue.Queue()
        self.modelingRunner = model_build.ModelingBuild(self.app)
        # signal for stopping modeling
        self.signalCancel.connect(self.modelingRunner.setCancel)

        # definitions for the command dispatcher. this enables spawning commands from outside into the current thread for running
        self.commandDispatch = {
            'RunInitialModel':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_runInitialModel,
                            'Method': self.modelingRunner.runInitialModel,
                            'Cancel': self.app.ui.btn_cancelInitialModel
                        }
                    ]
                },
            'RunFullModel':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_runFullModel,
                            'Method': self.modelingRunner.runFullModel,
                            'Cancel': self.app.ui.btn_cancelFullModel
                        }
                    ]
                },
            'PlateSolveSync':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_plateSolveSync,
                            'Method': self.modelingRunner.plateSolveSync
                        }
                    ]
                },
            'RunFlexure':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_runFlexure,
                            'Method': self.modelingRunner.runFlexure,
                            'Cancel': self.app.ui.btn_cancelAnalyse
                        }
                    ]
                },
            'RunHysterese':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_runHysterese,
                            'Method': self.modelingRunner.runHysterese,
                            'Cancel': self.app.ui.btn_cancelAnalyse
                        }
                    ]
                },
            'GenerateDSOPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateDSOPoints,
                            'Method': self.modelingRunner.modelPoints.generateDSOPoints,
                            'Parameter': ['self.app.ui.checkSortPoints.isChecked()',
                                          'int(self.app.ui.numberHoursDSO.value())',
                                          'int(self.app.ui.numberPointsDSO.value())',
                                          'int(self.app.ui.numberHoursPreview.value())'
                                          ]
                        }
                    ]
                },
            'GenerateMaxPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateMaxPoints,
                            'Method': self.modelingRunner.modelPoints.generateMaxPoints,
                            'Parameter': ['self.app.ui.checkDeletePointsHorizonMask.isChecked()',
                                          'self.app.ui.checkSortPoints.isChecked()'
                                          ]
                        }
                    ]
                },
            'GenerateNormalPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateNormalPoints,
                            'Method': self.modelingRunner.modelPoints.generateNormalPoints,
                            'Parameter': ['self.app.ui.checkDeletePointsHorizonMask.isChecked()',
                                          'self.app.ui.checkSortPoints.isChecked()'
                                          ]
                        }
                    ]
                },
            'GenerateMinPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateMinPoints,
                            'Method': self.modelingRunner.modelPoints.generateMinPoints,
                            'Parameter': ['self.app.ui.checkDeletePointsHorizonMask.isChecked()',
                                          'self.app.ui.checkSortPoints.isChecked()'
                                          ]
                        }
                    ]
                },
            'ShowInitialPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_showFullModelPoints,
                            'Method': self.modelingRunner.modelPoints.showInitialPoints,
                            'Parameter': ['self.app.ui.le_modelInitialPointsFileName.text()']
                        }
                    ]
                },
            'ShowFullPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_showFullModelPoints,
                            'Method': self.modelingRunner.modelPoints.showFullPoints,
                            'Parameter': ['self.app.ui.le_modelFullPointsFileName.text()',
                                          'self.app.ui.checkDeletePointsHorizonMask.isChecked()',
                                          'self.app.ui.checkSortPoints.isChecked()']
                        }
                    ]
                },
            'GenerateGridPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateGridPoints,
                            'Method': self.modelingRunner.modelPoints.generateGridPoints,
                            'Parameter': ['self.app.ui.checkDeletePointsHorizonMask.isChecked()',
                                          'self.app.ui.checkSortPoints.isChecked()',
                                          'int(self.app.ui.numberGridPointsRow.value())',
                                          'int(self.app.ui.numberGridPointsCol.value())',
                                          'int(self.app.ui.altitudeMin.value())',
                                          'int(self.app.ui.altitudeMax.value())']
                        }
                    ]
                },
            'GenerateInitialPoints':
                {
                    'Worker': [
                        {
                            'Button': self.app.ui.btn_generateInitialPoints,
                            'Method': self.modelingRunner.modelPoints.generateInitialPoints,
                            'Parameter': ['float(self.app.ui.azimuthBase.value())',
                                          'float(self.app.ui.altitudeBase.value())',
                                          'int(self.app.ui.numberBase.value())']
                        }
                    ]
                },
            'DeletePoints':
                {
                    'Worker': [
                        {
                            'Method': self.modelingRunner.modelPoints.deletePoints
                        }
                    ]
                }
            }
        # signal slot
        self.app.ui.btn_plateSolveSync.clicked.connect(lambda: self.commandDispatcherQueue.put('PlateSolveSync'))
        self.app.ui.btn_showFullModelPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('ShowFullPoints'))
        self.app.ui.btn_showInitialModelPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('ShowInitialPoints'))
        self.app.ui.btn_generateDSOPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberHoursDSO.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberPointsDSO.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberHoursPreview.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.btn_generateMaxPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateMaxPoints'))
        self.app.ui.btn_generateNormalPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateNormalPoints'))
        self.app.ui.btn_generateMinPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateMinPoints'))
        self.app.ui.btn_generateGridPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.numberGridPointsRow.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.numberGridPointsCol.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.altitudeMin.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.altitudeMax.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.btn_generateInitialPoints.clicked.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.altitudeBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.azimuthBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.numberBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.btn_runFlexure.clicked.connect(lambda: self.commandDispatcherQueue.put('RunFlexure'))
        self.app.ui.btn_runHysterese.clicked.connect(lambda: self.commandDispatcherQueue.put('RunHysterese'))
        self.app.ui.btn_runFullModel.clicked.connect(lambda: self.commandDispatcherQueue.put('RunFullModel'))
        self.app.ui.btn_runInitialModel.clicked.connect(lambda: self.commandDispatcherQueue.put('RunInitialModel'))

    def initConfig(self):
        # before changing value through config (which fires the signals) i have to disable them
        self.app.ui.numberHoursDSO.valueChanged.disconnect()
        self.app.ui.numberPointsDSO.valueChanged.disconnect()
        self.app.ui.numberHoursPreview.valueChanged.disconnect()
        self.app.ui.numberGridPointsRow.valueChanged.disconnect()
        self.app.ui.numberGridPointsCol.valueChanged.disconnect()
        self.app.ui.altitudeMin.valueChanged.disconnect()
        self.app.ui.altitudeMax.valueChanged.disconnect()
        self.app.ui.altitudeBase.valueChanged.disconnect()
        self.app.ui.azimuthBase.valueChanged.disconnect()
        self.app.ui.numberBase.valueChanged.disconnect()
        try:
            if 'CheckSortPoints' in self.app.config:
                self.app.ui.checkSortPoints.setChecked(self.app.config['CheckSortPoints'])
            if 'CheckDeletePointsHorizonMask' in self.app.config:
                self.app.ui.checkDeletePointsHorizonMask.setChecked(self.app.config['CheckDeletePointsHorizonMask'])
            if 'AltitudeBase' in self.app.config:
                self.app.ui.altitudeBase.setValue(self.app.config['AltitudeBase'])
            if 'AzimuthBase' in self.app.config:
                self.app.ui.azimuthBase.setValue(self.app.config['AzimuthBase'])
            if 'NumberGridPointsCol' in self.app.config:
                self.app.ui.numberGridPointsCol.setValue(self.app.config['NumberGridPointsCol'])
            if 'NumberGridPointsRow' in self.app.config:
                self.app.ui.numberGridPointsRow.setValue(self.app.config['NumberGridPointsRow'])
            if 'AltitudeMin' in self.app.config:
                self.app.ui.altitudeMin.setValue(self.app.config['AltitudeMin'])
            if 'AltitudeMax' in self.app.config:
                self.app.ui.altitudeMax.setValue(self.app.config['AltitudeMax'])
            if 'NumberPointsDSO' in self.app.config:
                self.app.ui.numberPointsDSO.setValue(self.app.config['NumberPointsDSO'])
            if 'NumberHoursDSO' in self.app.config:
                self.app.ui.numberHoursDSO.setValue(self.app.config['NumberHoursDSO'])

        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass
        self.modelingRunner.initConfig()
        # and restored
        self.app.ui.numberHoursDSO.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberPointsDSO.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberHoursPreview.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateDSOPoints'))
        self.app.ui.numberGridPointsRow.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.numberGridPointsCol.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.altitudeMin.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.altitudeMax.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateGridPoints'))
        self.app.ui.altitudeBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.azimuthBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))
        self.app.ui.numberBase.valueChanged.connect(lambda: self.commandDispatcherQueue.put('GenerateInitialPoints'))

    def storeConfig(self):
        self.app.config['CheckSortPoints'] = self.app.ui.checkSortPoints.isChecked()
        self.app.config['CheckDeletePointsHorizonMask'] = self.app.ui.checkDeletePointsHorizonMask.isChecked()
        self.app.config['AltitudeBase'] = self.app.ui.altitudeBase.value()
        self.app.config['AzimuthBase'] = self.app.ui.azimuthBase.value()
        self.app.config['NumberGridPointsRow'] = self.app.ui.numberGridPointsRow.value()
        self.app.config['NumberGridPointsCol'] = self.app.ui.numberGridPointsCol.value()
        self.app.config['AltitudeMin'] = self.app.ui.altitudeMin.value()
        self.app.config['AltitudeMax'] = self.app.ui.altitudeMax.value()
        self.app.config['NumberPointsDSO'] = self.app.ui.numberPointsDSO.value()
        self.app.config['NumberHoursDSO'] = self.app.ui.numberHoursDSO.value()
        # and calling the underlying classes as well
        self.modelingRunner.storeConfig()

    def run(self):
        self.logger.info('model dispatcher started')
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('model dispatcher stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.commandDispatcherQueue.empty():
            command = self.commandDispatcherQueue.get()
            self.commandDispatcher(command)

    def commandDispatcher(self, command):
        # if we have a command in dispatcher
        if command in self.commandDispatch:
            # running through all necessary commands
            for work in self.commandDispatch[command]['Worker']:
                # if we want to color a button, which one
                if 'Button' in work:
                    self.app.signalChangeStylesheet.emit(work['Button'], 'running', True)
                if 'Parameter' in work:
                    parameter = []
                    for p in work['Parameter']:
                        parameter.append(eval(p))
                    work['Method'](*parameter)
                else:
                    work['Method']()
                time.sleep(0.2)
                if 'Button' in work:
                    self.app.signalChangeStylesheet.emit(work['Button'], 'running', False)
                if 'Cancel' in work:
                    self.app.signalChangeStylesheet.emit(work['Cancel'], 'cancel', False)
                    self.modelingRunner.cancel = False



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.4
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import platform
import time
import PyQt5
if platform.system() == 'Windows':
    from dome import ascom_dome
from dome import indi_dome
from dome import none_dome


class Dome(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    signalDomeConnected = PyQt5.QtCore.pyqtSignal([int])
    signalDomePointer = PyQt5.QtCore.pyqtSignal(float, bool)
    signalSlewFinished = PyQt5.QtCore.pyqtSignal()
    domeStatusText = PyQt5.QtCore.pyqtSignal(str)
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    CYCLE = 200
    CYCLE_STATUS = 500

    def __init__(self, app, thread):
        super().__init__()
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.mutexChooser = PyQt5.QtCore.QMutex()
        self.dataTimer = None
        self.statusTimer = None
        self.cycleTimer = None

        self.app = app
        self.thread = thread
        self.data = {
            'Connected': False,
            'Slewing': False
        }
        # get supporting handlers
        if platform.system() == 'Windows':
            self.ascom = ascom_dome.AscomDome(self, self.app, self.data)
        self.indi = indi_dome.INDIDome(self, self.app, self.data)
        self.none = none_dome.NoneDome(self, self.app, self.data)
        # set handler to none
        self.domeHandler = self.none

        # connect change in dome to the subroutine of setting it up
        self.app.ui.pd_chooseDome.activated.connect(self.chooserDome)

    def initConfig(self):
        # first build the pull down menu
        self.app.ui.pd_chooseDome.clear()
        view = PyQt5.QtWidgets.QListView()
        self.app.ui.pd_chooseDome.setView(view)
        self.app.ui.pd_chooseDome.addItem('No Dome')
        if platform.system() == 'Windows':
            self.app.ui.pd_chooseDome.addItem('ASCOM')
        self.app.ui.pd_chooseDome.addItem('INDI')
        # load the config including pull down setup
        try:
            if platform.system() == 'Windows':
                if 'DomeAscomDriverName' in self.app.config:
                    self.ascom.driverName = self.app.config['DomeAscomDriverName']
                    self.app.ui.le_ascomDomeDriverName.setText(self.app.config['DomeAscomDriverName'])
            if 'Dome' in self.app.config:
                self.app.ui.pd_chooseDome.setCurrentIndex(int(self.app.config['Dome']))
        except Exception as e:
            self.logger.error('Item in config.cfg for dome could not be initialized, error:{0}'.format(e))
        finally:
            pass
        self.chooserDome()

    def storeConfig(self):
        if platform.system() == 'Windows':
            self.app.config['DomeAscomDriverName'] = self.ascom.driverName
        self.app.config['Dome'] = self.app.ui.pd_chooseDome.currentIndex()

    def chooserDome(self):
        self.mutexChooser.lock()
        self.stop()
        self.app.sharedDomeDataLock.lockForWrite()
        self.data['Connected'] = False
        self.app.sharedDomeDataLock.unlock()
        if self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            self.domeHandler = self.none
            self.logger.info('Actual dome is None')
        elif self.app.ui.pd_chooseDome.currentText().startswith('ASCOM'):
            self.domeHandler = self.ascom
            self.logger.info('Actual dome is ASCOM')
        elif self.app.ui.pd_chooseDome.currentText().startswith('INDI'):
            self.domeHandler = self.indi
            self.logger.info('Actual dome is INDI')
        if self.app.ui.pd_chooseDome.currentText().startswith('No Dome'):
            self.signalDomeConnected.emit(0)
        self.thread.start()
        self.mutexChooser.unlock()

    def run(self):
        self.logger.info('dome started')
        # a running thread is shown with variable isRunning = True. This thread should hav it's own event loop.
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.domeHandler.start()
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        # timers
        self.statusTimer = PyQt5.QtCore.QTimer(self)
        self.statusTimer.setSingleShot(False)
        self.statusTimer.timeout.connect(self.getStatusFromDevice)
        self.dataTimer = PyQt5.QtCore.QTimer(self)
        self.dataTimer.setSingleShot(False)
        self.dataTimer.timeout.connect(self.getDataFromDevice)
        self.statusTimer.start(self.CYCLE_STATUS)
        self.dataTimer.start(self.CYCLE_STATUS)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('dome stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.signalDestruct.connect(self.destruct)
        self.cycleTimer.stop()
        self.dataTimer.stop()
        self.statusTimer.stop()
        self.domeHandler.stop()

    def doCommand(self):
        if not self.app.domeCommandQueue.empty():
            command, value = self.app.domeCommandQueue.get()
            if command == 'SlewAzimuth':
                self.domeHandler.slewToAzimuth(value)

    @PyQt5.QtCore.pyqtSlot()
    def getStatusFromDevice(self):
        self.domeHandler.getStatus()
        # get status to gui
        if not self.domeHandler.application['Available']:
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_domeConnected, 'color', 'gray')
        elif self.domeHandler.application['Status'] == 'ERROR':
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_domeConnected, 'color', 'red')
        elif self.domeHandler.application['Status'] == 'OK':
            self.app.sharedDomeDataLock.lockForRead()
            if self.data['Connected'] == 'Off':
                self.app.signalChangeStylesheet.emit(self.app.ui.btn_domeConnected, 'color', 'yellow')
            else:
                self.app.signalChangeStylesheet.emit(self.app.ui.btn_domeConnected, 'color', 'green')
            self.app.sharedDomeDataLock.unlock()

    @PyQt5.QtCore.pyqtSlot()
    def getDataFromDevice(self):
        self.app.sharedDomeDataLock.lockForRead()
        connected = self.data['Connected']
        self.app.sharedDomeDataLock.unlock()
        if connected:
            self.domeHandler.getData()
        else:
            self.app.sharedDomeDataLock.lockForWrite()
            self.data['Slewing'] = False
            self.data['Azimuth'] = 0.0
            self.data['Altitude'] = 0.0
            self.app.sharedDomeDataLock.unlock()
        # signaling
        self.app.sharedDomeDataLock.lockForRead()
        if 'Azimuth' in self.data:
            self.signalDomePointer.emit(self.data['Azimuth'], self.data['Connected'])
        self.app.sharedDomeDataLock.unlock()



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import platform
import PyQt5
import time
import queue
import os
import astropy.io.fits as pyfits

from astrometry import client_astrometry
if platform.system() == 'Windows':
    from astrometry import sgpro_astrometry
    from astrometry import pinpoint_astrometry
from astrometry import none_astrometry
from astrometry import transform


class Astrometry(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    # signals to be used for others
    # putting status to gui
    astrometryStatusText = PyQt5.QtCore.pyqtSignal(str)
    astrometrySolvingTime = PyQt5.QtCore.pyqtSignal(str)
    astrometryCancel = PyQt5.QtCore.pyqtSignal()
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    # putting status to processing
    imageUploaded = PyQt5.QtCore.pyqtSignal()
    imageSolved = PyQt5.QtCore.pyqtSignal()
    imageDataDownloaded = PyQt5.QtCore.pyqtSignal()

    CYCLE = 200
    CYCLE_STATUS = 1000

    def __init__(self, app, thread):
        super().__init__()
        # make main sources available
        self.app = app
        self.thread = thread
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.astrometryCommandQueue = queue.Queue()
        self.mutexChooser = PyQt5.QtCore.QMutex()
        self.transform = transform.Transform(self.app)
        self.statusTimer = None
        self.cycleTimer = None

        # class data
        self.data = dict()
        self.data['CONNECTION'] = {'CONNECT': 'Off'}

        # external classes
        self.AstrometryClient = client_astrometry.AstrometryClient(self, self.app, self.data)
        if platform.system() == 'Windows':
            self.SGPro = sgpro_astrometry.SGPro(self, self.app, self.data)
            self.PinPoint = pinpoint_astrometry.PinPoint(self, self.app, self.data)
        self.NoneSolve = none_astrometry.NoneAstrometry(self, self.app, self.data)

        # set handler to default position
        self.astrometryHandler = self.NoneSolve

        # signal slot links
        self.astrometryCancel.connect(self.setCancelAstrometry)
        self.app.ui.pd_chooseAstrometry.activated.connect(self.chooseAstrometry)

    def initConfig(self):
        # build the drop down menu
        self.app.ui.pd_chooseAstrometry.clear()
        view = PyQt5.QtWidgets.QListView()
        self.app.ui.pd_chooseAstrometry.setView(view)
        if self.NoneSolve.application['Available']:
            self.app.ui.pd_chooseAstrometry.addItem('No Astrometry - ' + self.NoneSolve.application['Name'])
        if self.AstrometryClient.application['Available']:
            self.app.ui.pd_chooseAstrometry.addItem('Astrometry - ' + self.AstrometryClient.application['Name'])
        if platform.system() == 'Windows':
            if self.SGPro.application['Available']:
                self.app.ui.pd_chooseAstrometry.addItem('SGPro - ' + self.SGPro.application['Name'])
            if self.PinPoint.application['Available']:
                self.app.ui.pd_chooseAstrometry.addItem('PinPoint - ' + self.PinPoint.application['Name'])
        # if platform.system() == 'Windows' or platform.system() == 'Darwin':
        #    if self.workerTheSkyX.data['AppAvailable']:
        #        self.app.ui.pd_chooseAstrometry.addItem('TheSkyX - ' + self.workerTheSkyX.data['AppName'])
        # load the config data
        if 'PinPointCatalogue' in self.app.config:
            self.app.ui.le_pinpointCatalogue.setText(self.app.config['PinPointCatalogue'])
        try:
            if 'AstrometryApplication' in self.app.config:
                self.app.ui.pd_chooseAstrometry.setCurrentIndex(int(self.app.config['AstrometryApplication']))
        except Exception as e:
            self.logger.error('Item in config.cfg for astrometry could not be initialized, error:{0}'.format(e))
        finally:
            pass
        self.AstrometryClient.initConfig()
        if platform.system() == 'Windows':
            self.SGPro.initConfig()
        self.chooseAstrometry()

    def storeConfig(self):
        self.app.config['AstrometryApplication'] = self.app.ui.pd_chooseAstrometry.currentIndex()
        self.app.config['PinPointCatalogue'] = self.app.ui.le_pinpointCatalogue.text()
        if platform.system() == 'Windows':
            self.SGPro.storeConfig()
        self.AstrometryClient.storeConfig()

    def setCancelAstrometry(self):
        self.astrometryHandler.mutexCancel.lock()
        self.astrometryHandler.cancel = True
        self.astrometryHandler.mutexCancel.unlock()

    def chooseAstrometry(self):
        self.mutexChooser.lock()
        self.stop()
        if self.app.ui.pd_chooseAstrometry.currentText().startswith('No Astrometry'):
            self.astrometryHandler = self.NoneSolve
            self.logger.info('Actual plate solver is None')
        elif self.app.ui.pd_chooseAstrometry.currentText().startswith('SGPro'):
            self.astrometryHandler = self.SGPro
            self.logger.info('Actual plate solver is SGPro')
        elif self.app.ui.pd_chooseAstrometry.currentText().startswith('PinPoint'):
            self.astrometryHandler = self.PinPoint
            self.logger.info('Actual plate solver is PinPoint')
        elif self.app.ui.pd_chooseAstrometry.currentText().startswith('Astrometry'):
            self.astrometryHandler = self.AstrometryClient
            self.logger.info('Actual plate solver is ASTROMETRY.NET')
        elif self.app.ui.pd_chooseAstrometry.currentText().startswith('TheSkyX'):
            self.astrometryHandler = self.TheSkyX
            self.logger.info('Actual plate solver is TheSkyX')
        self.astrometryStatusText.emit('')
        if not self.astrometryHandler.application['Available']:
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'gray')
        else:
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'red')
        self.thread.start()
        self.mutexChooser.unlock()

    def run(self):
        self.logger.info('astrometry started')
        # a running thread is shown with variable isRunning = True. This thread should have it's own event loop.
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.astrometryHandler.start()
        # timers
        self.statusTimer = PyQt5.QtCore.QTimer(self)
        self.statusTimer.setSingleShot(False)
        self.statusTimer.timeout.connect(self.getStatusFromDevice)
        self.statusTimer.start(self.CYCLE_STATUS)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('astrometry stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.statusTimer.stop()
        self.astrometryHandler.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.astrometryCommandQueue.empty():
            imageParams = self.astrometryCommandQueue.get()
            self.solveImage(imageParams)

    def solveImage(self, imageParams):
        dataPresentForSolving = True
        if self.data['CONNECTION']['CONNECT'] == 'Off':
            return
        # reset message
        imageParams['Message'] = 'Cancelled'
        # check for use of FITS data
        if not os.path.isfile(imageParams['Imagepath']):
            return
        # open the fits header and check content.
        # we need for solving "OBJCTRA", "OBJCTDEC" and "PIXSCALE" field
        fitsFileHandle = pyfits.open(imageParams['Imagepath'])
        fitsHeader = fitsFileHandle[0].header
        if 'OBJCTRA' in fitsHeader:
            imageParams['RaJ2000'] = self.transform.degStringToDecimal(fitsHeader['OBJCTRA'], ' ')
        else:
            self.logger.error('FITS data OBJCTRA for start solving is missing, present headers: {0}'.format(fitsHeader))
            dataPresentForSolving = False
        if 'OBJCTDEC' in fitsHeader:
            imageParams['DecJ2000'] = self.transform.degStringToDecimal(fitsHeader['OBJCTDEC'], ' ')
        else:
            self.logger.error('FITS data OBJCTDEC for start solving is missing, present headers: {0}'.format(fitsHeader))
            dataPresentForSolving = False
        if 'PIXSCALE' in fitsHeader:
            imageParams['ScaleHint'] = fitsHeader['PIXSCALE']
        elif 'SCALE' in fitsHeader:
            imageParams['ScaleHint'] = fitsHeader['SCALE']
        else:
            self.logger.info('FITS data PIXSCALE for start solving is missing try to recalculate')
            # now trying to recalculate this value
            if 'FOCALLEN' in fitsHeader and 'XPIXSZ' in fitsHeader:
                if float(fitsHeader['FOCALLEN']) > 0:
                    imageParams['ScaleHint'] = float(fitsHeader['XPIXSZ']) * 206.6 / float(fitsHeader['FOCALLEN'])
                else:
                    self.logger.error('FITS data FOCALLEN is 0 -> cannot solve, present headers: {0}'.format(fitsHeader))
                    dataPresentForSolving = False
            elif 'FOCALLEN' in fitsHeader and 'PIXSIZE1' in fitsHeader and 'XBINNING' in fitsHeader:
                if float(fitsHeader['FOCALLEN']) > 0:
                    imageParams['ScaleHint'] = float(fitsHeader['PIXSIZE1']) * 206.6 / float(fitsHeader['FOCALLEN']) * float(fitsHeader['XBINNING'])
                else:
                    self.logger.error('FITS data FOCALLEN is 0 -> cannot solve, present headers: {0}'.format(fitsHeader))
                    dataPresentForSolving = False
            else:
                # if we cannot recalculate, there is no chance to get this parameter
                self.logger.error('FITS data FOCALLEN or XPIXSZ or PIXSIZE1 or XBINNING for start solving is missing, present headers: {0}'.format(fitsHeader))
                dataPresentForSolving = False
        fitsFileHandle.close()
        if dataPresentForSolving:
            self.logger.info('Params before solving: {0}'.format(imageParams))
            self.astrometryHandler.solveImage(imageParams)
            self.logger.info('Params after solving: {0}'.format(imageParams))
            if self.app.imageWindow.showStatus:
                if 'Solved' in imageParams:
                    if imageParams['Solved']:
                        self.app.imageWindow.signalSetRaSolved.emit(self.transform.decimalToDegree(imageParams['RaJ2000Solved'], False, False))
                        self.app.imageWindow.signalSetDecSolved.emit(self.transform.decimalToDegree(imageParams['DecJ2000Solved'], True, False))
                        self.app.imageWindow.signalSetAngleSolved.emit('{0:3.1f}'.format(imageParams['Angle']))
                    else:
                        self.app.imageWindow.signalSetRaSolved.emit('not solved')
                        self.app.imageWindow.signalSetDecSolved.emit('not solved')
                        self.app.imageWindow.signalSetAngleSolved.emit('-')
                else:
                    self.app.imageWindow.signalSetRaSolved.emit('not solved')
                    self.app.imageWindow.signalSetDecSolved.emit('not solved')
                    self.app.imageWindow.signalSetAngleSolved.emit('-')
        else:
            imageParams['Solved'] = False
            imageParams['Message'] = 'FITS header parameters missing'
            self.app.imageWindow.signalSetRaSolved.emit('no data')
            self.app.imageWindow.signalSetDecSolved.emit('no data')
            self.app.imageWindow.signalSetAngleSolved.emit('-')

    @PyQt5.QtCore.pyqtSlot()
    def getStatusFromDevice(self):
        # get status to gui
        if not self.astrometryHandler.application['Available']:
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'gray')
        else:
            self.astrometryHandler.getStatus()
            if self.astrometryHandler.application['Status'] == 'ERROR':
                self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'red')
            elif self.astrometryHandler.application['Status'] in ['OK', '']:
                if self.data['CONNECTION']['CONNECT'] == 'Off':
                    self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'yellow')
                else:
                    self.app.signalChangeStylesheet.emit(self.app.ui.btn_astrometryConnected, 'color', 'green')
            else:
                self.logger.warning('This state is undefined')

    def updateApplicationName(self):
        # updating solver name name if possible
        for i in range(0, self.app.ui.pd_chooseAstrometry.count()):
            if self.app.ui.pd_chooseAstrometry.itemText(i).startswith('No Astrometry'):
                pass
            elif self.app.ui.pd_chooseAstrometry.itemText(i).startswith('SGPro'):
                pass
            elif self.app.ui.pd_chooseAstrometry.itemText(i).startswith('MaximDL'):
                pass
            elif self.app.ui.pd_chooseAstrometry.itemText(i).startswith('Astrometry'):
                self.app.ui.pd_chooseAstrometry.setItemText(i, 'Astrometry - ' + self.AstrometryClient.application['Name'])
            elif self.app.ui.pd_chooseAstrometry.itemText(i).startswith('TheSkyX'):
                pass



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #  ####
#      ##  ##  #  ##  #     #
#     # # # #  # # # #     ###
#    #  ##  #  ##  ##        #
#   #   #   #  #   #     ####
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.6.5
#
# Michael Würtenberger
# (c) 2016, 2017, 2018
#
# Licence APL2.0
#
###########################################################
import logging
import os
import time
import platform
import PyQt5
import queue
import copy
import astropy.io.fits as pyfits
from astrometry import transform
from imaging import none_camera
from imaging import indi_camera
if platform.system() == 'Windows':
    from imaging import maximdl_camera
    from imaging import sgpro_camera
if platform.system() == 'Windows' or platform.system() == 'Darwin':
    from imaging import theskyx_camera


class Imaging(PyQt5.QtCore.QObject):
    logger = logging.getLogger(__name__)

    # signals to be used for others
    # putting status to gui
    cameraStatusText = PyQt5.QtCore.pyqtSignal(str)
    cameraExposureTime = PyQt5.QtCore.pyqtSignal(str)
    imagingCancel = PyQt5.QtCore.pyqtSignal()
    signalDestruct = PyQt5.QtCore.pyqtSignal()

    # putting status to processing
    imageIntegrated = PyQt5.QtCore.pyqtSignal()
    imageDownloaded = PyQt5.QtCore.pyqtSignal()
    imageSaved = PyQt5.QtCore.pyqtSignal()

    # where to place the images
    CYCLE = 200
    CYCLE_STATUS = 1000

    def __init__(self, app, thread):
        super().__init__()
        # make main sources available
        self.app = app
        self.thread = thread
        self.isRunning = False
        self.mutexIsRunning = PyQt5.QtCore.QMutex()
        self.imagingCommandQueue = queue.Queue()
        self.mutexChooser = PyQt5.QtCore.QMutex()
        self.mutexData = PyQt5.QtCore.QMutex()
        self.statusTimer = None
        self.cycleTimer = None
        self.IMAGEDIR = os.getcwd().replace('\\', '/') + '/images'

        # class data
        self.data = dict()
        self.data['CONNECTION'] = {'CONNECT': 'Off'}

        # external classes
        self.transform = transform.Transform(self.app)
        if platform.system() == 'Windows':
            self.SGPro = sgpro_camera.SGPro(self, self.app, self.data)
            self.MaximDL = maximdl_camera.MaximDL(self, self.app, self.data)
        self.INDICamera = indi_camera.INDICamera(self, self.app, self.data)
        self.NoneCam = none_camera.NoneCamera(self, self.app, self.data)

        # set the camera handler to default position
        self.cameraHandler = self.NoneCam

        # signal slot links
        self.imagingCancel.connect(self.setCancelImaging)
        self.app.ui.pd_chooseImaging.activated.connect(self.chooseImaging)

    def initConfig(self):
        # build the drop down menu
        self.app.ui.pd_chooseImaging.clear()
        view = PyQt5.QtWidgets.QListView()
        self.app.ui.pd_chooseImaging.setView(view)
        if self.NoneCam.application['Available']:
            self.app.ui.pd_chooseImaging.addItem('No Camera - ' + self.NoneCam.application['Name'])
        if self.INDICamera.application['Available']:
            self.app.ui.pd_chooseImaging.addItem('INDI Camera - ' + self.INDICamera.application['Name'])
        if platform.system() == 'Windows':
            if self.SGPro.application['Available']:
                self.app.ui.pd_chooseImaging.addItem('SGPro - ' + self.SGPro.application['Name'])
            if self.MaximDL.application['Available']:
                self.app.ui.pd_chooseImaging.addItem('MaximDL - ' + self.MaximDL.application['Name'])
        # if platform.system() == 'Windows' or platform.system() == 'Darwin':
        #    if self.workerTheSkyX.data['AppAvailable']:
        #        self.app.ui.pd_chooseImaging.addItem('TheSkyX - ' + self.workerTheSkyX.data['AppName'])
        # load the config data
        try:
            if 'ImagingApplication' in self.app.config:
                self.app.ui.pd_chooseImaging.setCurrentIndex(int(self.app.config['ImagingApplication']))
        except Exception as e:
            self.logger.error('item in config.cfg not be initialize, error:{0}'.format(e))
        finally:
            pass
        self.chooseImaging()

    def storeConfig(self):
        self.app.config['ImagingApplication'] = self.app.ui.pd_chooseImaging.currentIndex()

    def setCancelImaging(self):
        self.cameraHandler.mutexCancel.lock()
        self.cameraHandler.cancel = True
        self.cameraHandler.mutexCancel.unlock()

    def chooseImaging(self):
        self.mutexChooser.lock()
        self.stop()
        if self.app.ui.pd_chooseImaging.currentText().startswith('No Camera'):
            self.cameraHandler = self.NoneCam
            self.logger.info('Actual camera is None')
        elif self.app.ui.pd_chooseImaging.currentText().startswith('SGPro'):
            self.cameraHandler = self.SGPro
            self.logger.info('Actual camera is SGPro')
        elif self.app.ui.pd_chooseImaging.currentText().startswith('MaximDL'):
            self.cameraHandler = self.MaximDL
            self.logger.info('Actual camera is MaximDL')
        elif self.app.ui.pd_chooseImaging.currentText().startswith('INDI'):
            self.cameraHandler = self.INDICamera
            self.logger.info('Actual camera is INDI Camera')
        elif self.app.ui.pd_chooseImaging.currentText().startswith('TheSkyX'):
            self.cameraHandler = self.TheSkyX
            self.logger.info('Actual camera is TheSkyX')
        self.cameraStatusText.emit('')
        self.thread.start()
        self.mutexChooser.unlock()

    def run(self):
        self.logger.info('imaging started')
        # a running thread is shown with variable isRunning = True. This thread should have it's own event loop.
        self.mutexIsRunning.lock()
        if not self.isRunning:
            self.isRunning = True
        self.mutexIsRunning.unlock()
        self.cameraHandler.start()
        # timers
        self.statusTimer = PyQt5.QtCore.QTimer(self)
        self.statusTimer.setSingleShot(False)
        self.statusTimer.timeout.connect(self.getStatusFromDevice)
        self.statusTimer.start(self.CYCLE_STATUS)
        self.signalDestruct.connect(self.destruct, type=PyQt5.QtCore.Qt.BlockingQueuedConnection)
        self.cycleTimer = PyQt5.QtCore.QTimer(self)
        self.cycleTimer.setSingleShot(False)
        self.cycleTimer.timeout.connect(self.doCommand)
        self.cycleTimer.start(self.CYCLE)

    def stop(self):
        self.mutexIsRunning.lock()
        if self.isRunning:
            self.isRunning = False
            self.signalDestruct.emit()
            self.thread.quit()
            self.thread.wait()
        self.mutexIsRunning.unlock()
        self.logger.info('imaging stopped')

    @PyQt5.QtCore.pyqtSlot()
    def destruct(self):
        self.cycleTimer.stop()
        self.statusTimer.stop()
        self.cameraHandler.stop()
        self.signalDestruct.disconnect(self.destruct)

    def doCommand(self):
        if not self.imagingCommandQueue.empty():
            imageParams = self.imagingCommandQueue.get()
            self.captureImage(imageParams)

    def captureImage(self, imageParams):
        imageParams['Imagepath'] = ''
        if self.cameraHandler.application['Status'] != 'OK':
            imageParams['Imagepath'] = 'False'
            return
        # preparation for imaging: gathering all the informations for taking a picture from gui
        imageParams['Imagepath'] = ''
        if self.data['CONNECTION']['CONNECT'] == 'Off':
            imageParams['Imagepath'] = 'False'
            return
        self.cameraHandler.getCameraProps()
        imageParams['BaseDirImages'] = self.IMAGEDIR + '/' + imageParams['Directory']
        if not os.path.isdir(imageParams['BaseDirImages']):
            os.makedirs(imageParams['BaseDirImages'])
        imageParams['Binning'] = int(self.app.ui.cameraBin.value())
        imageParams['Exposure'] = int(self.app.ui.cameraExposure.value())
        imageParams['Iso'] = int(self.app.ui.isoSetting.value())
        if self.app.ui.checkDoSubframe.isChecked():
            scaleSubframe = self.app.ui.scaleSubframe.value() / 100
            imageParams['SizeX'] = int(self.data['CCD_INFO']['CCD_MAX_X']) * scaleSubframe
            imageParams['SizeY'] = int(self.data['CCD_INFO']['CCD_MAX_Y']) * scaleSubframe
            imageParams['OffX'] = int((float(self.data['CCD_INFO']['CCD_MAX_X']) - imageParams['SizeX']) / 2)
            imageParams['OffY'] = int((float(self.data['CCD_INFO']['CCD_MAX_Y']) - imageParams['SizeY']) / 2)
            imageParams['CanSubframe'] = True
        else:
            imageParams['SizeX'] = int(self.data['CCD_INFO']['CCD_MAX_X'])
            imageParams['SizeY'] = int(self.data['CCD_INFO']['CCD_MAX_Y'])
            imageParams['OffX'] = 0
            imageParams['OffY'] = 0
            imageParams['CanSubframe'] = False
        if 'Gain' in self.data:
            imageParams['Gain'] = self.data['Gain']
        else:
            imageParams['Gain'] = 'NotSet'
        if self.app.ui.checkFastDownload.isChecked():
            imageParams['Speed'] = 'HiSpeed'
        else:
            imageParams['Speed'] = 'Normal'
        # setting mount conditions for the taken image
        if 'RaJ2000' not in self.app.workerMountDispatcher.data:
            return
        imageParams['LocalSiderealTime'] = copy.copy(self.app.workerMountDispatcher.data['LocalSiderealTime'])
        imageParams['LocalSiderealTimeFloat'] = self.transform.degStringToDecimal(self.app.workerMountDispatcher.data['LocalSiderealTime'][0:9])
        imageParams['RaJ2000'] = copy.copy(self.app.workerMountDispatcher.data['RaJ2000'])
        imageParams['DecJ2000'] = copy.copy(self.app.workerMountDispatcher.data['DecJ2000'])
        imageParams['RaJNow'] = copy.copy(self.app.workerMountDispatcher.data['RaJNow'])
        imageParams['DecJNow'] = copy.copy(self.app.workerMountDispatcher.data['DecJNow'])
        imageParams['Pierside'] = copy.copy(self.app.workerMountDispatcher.data['Pierside'])
        imageParams['RefractionTemperature'] = copy.copy(self.app.workerMountDispatcher.data['RefractionTemperature'])
        imageParams['RefractionPressure'] = copy.copy(self.app.workerMountDispatcher.data['RefractionPressure'])
        self.logger.info('Params before imaging: {0}'.format(imageParams))
        # now we take the picture
        self.cameraHandler.getImage(imageParams)
        # if we got an image, than we work with it
        if os.path.isfile(imageParams['Imagepath']):
            # add the coordinates to the image of the telescope if not present
            # problem is the variety of definitions and fields, which could be used
            # find e.g. https://heasarc.gsfc.nasa.gov/docs/fcg/common_dict.html
            fitsFileHandle = pyfits.open(imageParams['Imagepath'], mode='update')
            fitsHeader = fitsFileHandle[0].header
            newRA = copy.copy(imageParams['RaJ2000'])
            newDEC = copy.copy(imageParams['DecJ2000'])
            newRAhms = self.transform.decimalToDegree(imageParams['RaJ2000'], False, True, ' ')
            newDEChms = self.transform.decimalToDegree(imageParams['DecJ2000'], True, True, ' ')
            # if we are missing coordinates, we are replacing them with actual data from mount
            if 'OBJCTRA' not in fitsHeader:
                self.logger.warning('No OBJCTRA in FITS Header, writing now: {0}'.format(newRA))
            else:
                self.logger.info('OBJCTRA in header was: {0}, writing now: {1}'.format(fitsHeader['OBJCTRA'], newRAhms))
            if 'OBJCTDEC' not in fitsHeader:
                self.logger.warning('No OBJCTDEC in FITS Header, writing now {0}'.format(newDEC))
            else:
                self.logger.info('OBJCTDEC in header was: {0}, writing now: {1}'.format(fitsHeader['OBJCTDEC'], newDEChms))
            # setting coordinates explicit, because MW does slewing after imaging and MW does not know, when imaging application takes coordinates from mount driver
            fitsHeader['OBJCTRA'] = newRAhms
            fitsHeader['OBJCTDEC'] = newDEChms
            # other used header entries by SGPro
            if 'RA' in fitsHeader:
                del fitsHeader['RA']
            if 'DEC' in fitsHeader:
                del fitsHeader['DEC']
            if 'CRVAL1' in fitsHeader:
                del fitsHeader['CRVAL1']
            if 'CRVAL2' in fitsHeader:
                del fitsHeader['CRVAL2']
            # if optical system data is missing in header, we replace them with data from GUI of mountwizzard
            if 'FOCALLEN' not in fitsHeader:
                fitsHeader['FOCALLEN'] = self.app.ui.focalLength.value()
                self.logger.warning('No FOCALLEN in FITS Header, writing')
            if 'XPIXSZ' not in fitsHeader:
                fitsHeader['XPIXSZ'] = self.app.ui.pixelSize.value() * self.app.ui.cameraBin.value()
                self.logger.warning('No XPIXSZ in FITS Header, writing')
            if 'PIXSIZE1' not in fitsHeader:
                fitsHeader['PIXSIZE1'] = self.app.ui.pixelSize.value()
                self.logger.warning('No PIXSIZE1 in FITS Header, writing')
            if 'YPIXSZ' not in fitsHeader:
                fitsHeader['YPIXSZ'] = self.app.ui.pixelSize.value() * self.app.ui.cameraBin.value()
                self.logger.warning('No YPIXSZ in FITS Header, writing')
            if 'PIXSIZE2' not in fitsHeader:
                fitsHeader['PIXSIZE2'] = self.app.ui.pixelSize.value()
                self.logger.warning('No PIXSIZE2 in FITS Header, writing')
            if 'XBINNING' not in fitsHeader:
                fitsHeader['XBINNING'] = self.app.ui.cameraBin.value()
                self.logger.warning('No XBINNING in FITS Header, writing')
            # refreshing FITS file with that data
            fitsFileHandle.flush()
            fitsFileHandle.close()
        else:
            pass
        # now imaging process is finished and told to everybody
        self.imageSaved.emit()
        self.data['Imaging'] = False
        # show it
        self.app.imageWindow.signalShowFitsImage.emit(imageParams['Imagepath'])

    @PyQt5.QtCore.pyqtSlot()
    def getStatusFromDevice(self):
        self.cameraHandler.getStatus()
        # get status to gui
        if not self.cameraHandler.application['Available']:
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_cameraConnected, 'color', 'gray')
        elif self.cameraHandler.application['Status'] == 'ERROR':
            self.app.signalChangeStylesheet.emit(self.app.ui.btn_cameraConnected, 'color', 'red')
        elif self.cameraHandler.application['Status'] == 'OK':
            if self.data['CONNECTION']['CONNECT'] == 'Off':
                self.app.signalChangeStylesheet.emit(self.app.ui.btn_cameraConnected, 'color', 'yellow')
            else:
                self.app.signalChangeStylesheet.emit(self.app.ui.btn_cameraConnected, 'color', 'green')

    def updateApplicationName(self):
        # updating camera name if possible
        for i in range(0, self.app.ui.pd_chooseImaging.count()):
            if self.app.ui.pd_chooseImaging.itemText(i).startswith('No Camera'):
                pass
            elif self.app.ui.pd_chooseImaging.itemText(i).startswith('SGPro'):
                pass
            elif self.app.ui.pd_chooseImaging.itemText(i).startswith('MaximDL'):
                pass
            elif self.app.ui.pd_chooseImaging.itemText(i).startswith('INDI'):
                self.app.ui.pd_chooseImaging.setItemText(i, 'INDI Camera - ' + self.INDICamera.application['Name'])
            elif self.app.ui.pd_chooseImaging.itemText(i).startswith('TheSkyX'):
                pass



############################################################
# -*- coding: utf-8 -*-
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.5
#
# Michael Würtenberger
# (c) 2016, 2017
#
# Licence APL2.0
#
############################################################
import logging
from PyQt5 import QtCore, QtWidgets
import sys
import time
import PyQt5

import numpy
import astropy.io.fits as pyfits
# import .NET / COM Handling
from win32com.client import Dispatch
import pythoncom


class Camera(QtCore.QObject):
    # in minimum there should be the logger in an the finished signal for clean-up
    logger = logging.getLogger(__name__)
    finished = QtCore.pyqtSignal()

    exposeSignal = QtCore.pyqtSignal(object)
    statusSignal = QtCore.pyqtSignal(str)

    # cycle time for recurrent task which are managed via QTimer
    CYCLE_PROPS = 500

    def __init__(self):
        # the main part and variables for the threading with dedicated event loop
        super().__init__()
        self.isRunning = False
        self._mutex = QtCore.QMutex()

        # self.driverName = 'QSICamera.CCDCamera'
        self.driverName = 'ASCOM.Simulator.Camera'
        self.ascom = None
        self.status = ''
        self.data = {}

    def run(self):
        # a running thread is shown with variable isRunning = True. This thread should hav it's own event loop.
        if not self.isRunning:
            self.isRunning = True
        if self.driverName != '':
            pythoncom.CoInitialize()
            try:
                self.ascom = Dispatch(self.driverName)
                self.ascom.connected = True
            except Exception as e:
                # self.logger.error('Could not dispatch driver: {0} and connect it. Stopping thread.'.format(self.driverName))
                print('could not dispatch', e)
            finally:
                pass
            # now starting all the tasks for cyclic doing (the ones which rely on QTimer)
            self.getProps()
        else:
            self.isRunning = False
        # main loop, if there is something to do, it should be inside. Important: all functions should be non blocking or calling processEvents()
        while self.isRunning:
            time.sleep(0.2)
            PyQt5.QtWidgets.QApplication.processEvents()
        # when the worker thread finished, it emit the finished signal to the parent to clean up
        self.finished.emit()
        self.ascom = None
        pythoncom.CoUninitialize()

    def stop(self):
        self._mutex.lock()
        self.isRunning = False
        self._mutex.unlock()

    def getProps(self):
        if self.isRunning:
            print('stat')
            self.data['CameraState'] = self.ascom.CameraState
            self.data['CameraXSize'] = self.ascom.CameraXSize
            self.data['CameraYSize'] = self.ascom.CameraYSize
            '''
            self.data['CanAbortExposure'] = self.ascom.CanAbortExposure
            self.data['CanFastReadout'] = self.ascom.CanFastReadout
            self.data['CanStopExposure'] = self.ascom.CanStopExposure
            self.data['CanGetCoolerPower'] = self.ascom.CanGetCoolerPower
            self.data['CCDTemperature'] = self.ascom.CCDTemperature
            self.data['Gain'] = self.ascom.Gain
            self.data['Gains'] = self.ascom.Gains
            self.data['InterfaceVersion'] = self.ascom.InterfaceVersion
            self.data['MaxBinX'] = self.ascom.MaxBinX
            self.data['MaxBinY'] = self.ascom.MaxBinY
            self.data['PixelSizeX'] = self.ascom.PixelSizeX
            self.data['PixelSizeY'] = self.ascom.PixelSizeY
            self.data['ReadoutMode'] = self.ascom.ReadoutMode
            self.data['ReadoutModes'] = self.ascom.ReadoutModes
            '''
            if self.data['CameraState'] in [0]:
                self.statusSignal.emit('IDLE')
            elif self.data['CameraState'] in [1, 2, 3]:
                self.statusSignal.emit('INTEGRATING')
            elif self.data['CameraState'] in [4]:
                self.statusSignal.emit('DOWNLOADING')
            else:
                self.statusSignal.emit('ERROR')
            QtCore.QTimer.singleShot(self.CYCLE_PROPS, self.getProps)

    def expose(self):
        if self.isRunning:
            try:
                self.ascom.BinX = 1
                self.ascom.BinY = 1
                self.ascom.NumX = 3388
                self.ascom.NumY = 2712
                self.ascom.StartX = 0
                self.ascom.StartY = 0
                # self.ascomCamera.Gains = modelData['gainValue']
                if self.ascom.CameraState == 0:
                    print('exposing', QtCore.QThread.currentThread())
                    self.ascom.StartExposure(3, True)
                else:
                    print('no exposure possible')
                    return
                while not self.ascom.ImageReady:
                    QtWidgets.QApplication.processEvents()
                print('exposing finished')
                # self.ascomCamera.ReadoutModes = modelData['speed']
                '''
                image = numpy.rot90(numpy.array(self.ascomCamera.ImageArray))
                image = numpy.flipud(image)
                hdu = pyfits.PrimaryHDU(image)
                imageData['imagepath'] = imageData['base_dir_images'] + '/' + imageData['file']
                hdu.writeto(imageData['imagepath'])
                '''
            except Exception as e:
                print(e)

    def setupDriverCamera(self):
        try:
            chooser = Dispatch('ASCOM.Utilities.Chooser')
            chooser.DeviceType = 'Camera'
            driverName = chooser.Choose('')
            print(driverName)
            self.connected = False
        except Exception as e:
            pass
        finally:
            pass


if __name__ == "__main__":

    class Widget(QtWidgets.QWidget):

        def __init__(self):
            QtWidgets.QWidget.__init__(self)
            self.button = QtWidgets.QPushButton('Stop', self)

            self.buttonE = QtWidgets.QPushButton('Expose', self)
            self.state = QtWidgets.QPushButton('State', self)
            layout = QtWidgets.QVBoxLayout(self)
            layout.addWidget(self.button)
            layout.addWidget(self.buttonE)
            layout.addWidget(self.state)

            self.worker = Camera()
            self.thread = QtCore.QThread()
            self.worker.moveToThread(self.thread)
            self.thread.started.connect(self.worker.run)
            self.worker.finished.connect(self.workerStop)
            self.thread.start()
            self.button.clicked.connect(self.worker.stop)
            # self.worker.expose tuns in context of the new thread
            self.buttonE.clicked.connect(self.worker.expose)
            self.worker.statusSignal.connect(self.status)

        def status(self, wert):
            self.state.setText(str(wert))

        def workerStop(self):
            self.thread.quit()
            self.thread.wait()

    app = QtWidgets.QApplication(sys.argv)
    widget = Widget()
    widget.show()
    widget.worker.stop()
    for i in range(0, 100):
        print(i)
        widget.thread.start()
        time.sleep(0.1)
        widget.worker.stop()

    while widget.worker.isRunning:
        QtWidgets.QApplication.processEvents()

    sys.exit(app.exec_())



# from distutils.core import setup
from setuptools import setup
import mountwizzard3.build.build
import platform

setup(
    name='mountwizzard3',
    version=mountwizzard3.build.build.BUILD().BUILD_NO_FILE,
    packages=[
        'mountwizzard3',
        'mountwizzard3.analyse',
        'mountwizzard3.ascom',
        'mountwizzard3.astrometry',
        'mountwizzard3.audio',
        'mountwizzard3.build',
        'mountwizzard3.automation',
        'mountwizzard3.baseclasses',
        'mountwizzard3.imaging',
        'mountwizzard3.dome',
        'mountwizzard3.environment',
        'mountwizzard3.gui',
        'mountwizzard3.icons',
        'mountwizzard3.indi',
        'mountwizzard3.modeling',
        'mountwizzard3.mount',
        'mountwizzard3.relays',
        'mountwizzard3.remote',
        'mountwizzard3.widgets'
    ],
    python_requires='~=3.6.3',
    install_requires=[
        'PyQt5==5.11.2',
        'matplotlib==2.2.2',
        'wakeonlan>=1.0.0',
        'requests==2.18.4',
        'astropy==3.0.3',
        'numpy==1.15.2',
        'requests_toolbelt==0.8.0'
    ]
    + (['pypiwin32==220'] if "Windows" == platform.system() else [])
    + (['pywinauto==0.6.4'] if "Windows" == platform.system() else [])
    + (['comtypes==1.1.1'] if "Windows" == platform.system() else [])
    ,
    url='https://github.com/mworion/MountWizzard3-DIST',
    license='APL 2.0',
    author='mw',
    author_email='michael@wuertenberger.org',
    description='tooling for a 10micron mount',
)

if platform.system() == 'Linux':
    print('#############################################')
    print('### Important hint:                       ###')
    print('### There might be the need to install    ###')
    print('### libfreetype6-dev manually as well     ###')
    print('### sudo apt-get install libfreetype6-dev ###')
    print('#############################################')

